import {
  createHTTPRequestHandler,
  createStaticBuildAdapter,
  handleRequest
} from "./chunk.KAXTYKCQ.js";

// src/shared/polyfills.ts
async function installURLPattern() {
  if (!globalThis.URLPattern) {
    const { URLPattern: URLPattern2 } = await import("urlpattern-polyfill/urlpattern");
    globalThis.URLPattern = URLPattern2;
  }
}

// src/shared/utils/date.ts
var formatDateString = (str, defaultDateString = "") => {
  const dateMatch = str.match(/\b(\d{4})-(\d{1,2})-(\d{1,2})\b/);
  if (dateMatch === null) {
    return defaultDateString;
  }
  const [, yearStr, monthStr, dayStr] = dateMatch;
  return [yearStr, monthStr.padStart(2, "0"), dayStr.padStart(2, "0")].join(
    "-"
  );
};

// src/shared/utils/error.ts
function coalesceToError(err) {
  return err instanceof Error || err && err.name && err.message ? err : new Error(JSON.stringify(err));
}

// src/shared/utils/html.ts
var escapeMap = {
  "&": "&amp;",
  "<": "&lt;",
  ">": "&gt;",
  "'": "&#39;",
  '"': "&quot;"
};
var escapeHtmlRE = /[&<>'"]/g;
function escapeHTML(str) {
  return str.replace(escapeHtmlRE, (char) => escapeMap[char]);
}
var unescapeMap = Object.fromEntries(
  Object.entries(escapeMap).map((entries) => entries.reverse())
);
var unescapeHtmlRE = /&amp;|&lt;|&gt;|&#39;|&quot;/g;
function unescapeHTML(str) {
  return str.replace(unescapeHtmlRE, (char) => unescapeMap[char]);
}

// src/shared/utils/json.ts
var prettyJsonStr = (obj) => JSON.stringify(obj, void 0, 2);
var stripImportQuotesRE = /"\(\) => import\((.+)\)"/g;
function stripImportQuotesFromJson(json) {
  return json.replace(stripImportQuotesRE, `() => import($1)`);
}
function fixRegexInPath(filePath) {
  return filePath.replace(/\(.*\)/, (m) => m.replace(/\//g, "\\"));
}

// src/shared/utils/unit.ts
function noop() {
}
function safeNotEqual(a, b) {
  return a != a ? b == b : a !== b || a && typeof a === "object" || typeof a === "function";
}
function isUndefined(value) {
  return typeof value === "undefined";
}
function isNull(value) {
  return value === null;
}
function isNumber(value) {
  return (value == null ? void 0 : value.constructor) === Number && !Number.isNaN(value);
}
function isString(value) {
  return (value == null ? void 0 : value.constructor) === String;
}
function isBoolean(value) {
  return (value == null ? void 0 : value.constructor) === Boolean;
}
function isArray(value) {
  return Array.isArray(value);
}
function isFunction(value) {
  return typeof value === "function";
}
var isObject = (val) => Object.prototype.toString.call(val) === "[object Object]";

// src/shared/utils/url.ts
var EXTERNAL_URL_RE = /^https?:/i;
var endslash = (str) => /(\.html|\/)$/.test(str) ? str : str + "/";
var slashSplitRE = /(?=\/)/g;
var slashedSplit = (path13) => slash(path13).split(slashSplitRE);
var slash = (str) => str.replace(/^\/?/, "/");
var noslash = (str) => str.replace(/^\//, "");
var noendslash = (str) => str.replace(/\/$/, "");
var slashes = (str) => slash(noendslash(str));
var isLinkHttp = (link) => /^(https?:)?\/\//.test(link);
var isLinkExternal = (link, base = "/") => {
  if (isLinkHttp(link)) {
    return true;
  }
  if (link.startsWith("/") && !link.startsWith(base)) {
    return true;
  }
  return false;
};
var isLinkMailto = (link) => /^mailto:/.test(link);
var isLinkTel = (link) => /^tel:/.test(link);
var HASH_RE = /#.*$/s;
var QUERY_RE = /\?.*$/s;
var cleanUrl = (url) => url.replace(HASH_RE, "").replace(QUERY_RE, "");
var resolveRoutePathFromUrl = (url, baseUrl = "/") => url.replace(/^(https?:)?\/\/[^/]*/, "").replace(new RegExp(`^${baseUrl}`), "/");

// src/shared/utils/page.ts
var STRIP_PAGE_ORDER_RE = /\/\[(\d*)\]/g;
function stripPageOrderFromPath(filePath) {
  return filePath.replace(STRIP_PAGE_ORDER_RE, "/");
}
function calcPageOrderScore(filePath) {
  let score = 1;
  for (const matches of filePath.matchAll(STRIP_PAGE_ORDER_RE) ?? []) {
    score *= Number(matches[1]);
  }
  return score;
}
function sortOrderedPageFiles(files) {
  return files.map(slash).sort(
    (fileA, fileB) => calcPageOrderScore(fileA) - calcPageOrderScore(fileB)
  ).map(stripPageOrderFromPath);
}
function isLoadedPage(page) {
  return typeof page === "object" && "$$loaded" in page;
}
function isLoadedMarkdownPage(page) {
  return isLoadedPage(page) && page.ext === ".md";
}
var splitRE = /\//g;
var htmlExtRE = /(?:index)?\.html$/;
var DATA_ASSET_BASE_PATH = "/_immutable/data";
function resolveDataAssetID(pathname, layoutIndex) {
  const id = `${isNumber(layoutIndex) ? `$${layoutIndex}:` : ""}${pathname.replace(splitRE, "_").replace(htmlExtRE, "")}`;
  return id;
}
var layoutIndexRE = /^\/?\$\d+/;
function parseDataAssetURL(url) {
  const pathname = noslash(
    decodeURI(url.pathname).replace(DATA_ASSET_BASE_PATH, "")
  );
  const [layoutIndex, route] = layoutIndexRE.test(pathname) ? pathname.slice(1).split(":") : [-1, pathname];
  let routePath = route.replace(/_/g, "/").replace(/\.json$/, "");
  if (!routePath.endsWith("/"))
    routePath += ".html";
  return {
    url: new URL(routePath, url.href),
    layoutIndex: Number(layoutIndex)
  };
}

// src/shared/utils/route.ts
var PATH_SCORE = {
  Segment: 6,
  Static: 5,
  Group: 3,
  Dynamic: 2,
  BonusRegExp: 1,
  BonusExt: 3,
  PenaltyOptional: -4,
  PenaltyWildcard: -7,
  PenaltyRepeatable: -8,
  PenaltyRegexWildcard: -10
};
var bonusRegexRE = /\(.*?\)/;
var bonusExtRE = /\.html$/;
var penaltyOptionalRE = /(^\/:(\w|\.)+\?)|({:?.*?}\?)|(\(.*?\)\?)/;
var penaltyWildcardRE = /^\/(\*|(:?\w*?\*))/;
var penaltyRepeatableRE = /^\/(:(\w|\.)+(\+|\*))|({:?.*?}(\+|\*))/;
var penaltyRegexWildcardRE = /(\(\.\*\??\)|(\.\+\??))/;
var scoreCache = /* @__PURE__ */ new Map();
function calcRoutePathScore(pathname) {
  if (scoreCache.has(pathname))
    return scoreCache.get(pathname);
  const segments = splitRoutePath(pathname);
  let score = 0;
  for (const segment of segments) {
    const isDynamic = isRoutePathDynamic(segment);
    const isNamedGroup = segment.startsWith("/:") || segment.startsWith("/{:");
    let isSegment = true;
    if (bonusExtRE.test(segment)) {
      score += PATH_SCORE.BonusExt;
    }
    if (isNamedGroup) {
      score += PATH_SCORE.Group;
    } else if (isDynamic) {
      score += PATH_SCORE.Dynamic;
    }
    if (isNamedGroup || isDynamic) {
      if (bonusRegexRE.test(segment)) {
        score += PATH_SCORE.BonusRegExp;
      }
      if (penaltyOptionalRE.test(segment)) {
        score += PATH_SCORE.PenaltyOptional;
        isSegment = false;
      }
      if (penaltyWildcardRE.test(segment)) {
        score += PATH_SCORE.PenaltyWildcard;
        isSegment = false;
      }
      if (penaltyRepeatableRE.test(segment)) {
        score += PATH_SCORE.PenaltyRepeatable;
        isSegment = false;
      }
      if (penaltyRegexWildcardRE.test(segment)) {
        score += PATH_SCORE.PenaltyRegexWildcard;
        isSegment = false;
      }
    } else {
      score += PATH_SCORE.Static;
    }
    if (isSegment)
      score += PATH_SCORE.Segment;
  }
  scoreCache.set(pathname, score);
  return score;
}
function compareRoutes(routeA, routeB) {
  if (routeA.score !== routeB.score) {
    return routeB.score - routeA.score;
  }
  const segmentsA = splitRoutePath(routeA.pathname).length;
  const segmentsB = splitRoutePath(routeB.pathname).length;
  return segmentsA != segmentsB ? segmentsB - segmentsA : routeB.pathname.length - routeA.pathname.length;
}
var trailingHtmlExtGroupRE = /{\.html}\?$/;
var trailingSlashGroupRE = /{\/}\?{index}\?{\.html}\?$/;
function cleanRoutePath(pathname) {
  return pathname.replace(trailingSlashGroupRE, "/index.html").replace(trailingHtmlExtGroupRE, ".html");
}
var routeSplitRE = /\/(?![^{(]*}|\))/g;
function splitRoutePath(pathname) {
  return noslash(cleanRoutePath(pathname)).split(routeSplitRE).map(slash);
}
var dynamicPathRE = /(\*|\(.*\)|{.*}|\/:)/;
function isRoutePathDynamic(pathname) {
  return pathname.startsWith("/:") || dynamicPathRE.test(cleanRoutePath(pathname));
}
function matchRoute(url, routes) {
  const result = matchRouteInfo(url, routes);
  return result ? { ...result.route, match: result.match } : void 0;
}
function matchRouteInfo(url, routes) {
  const normalized = "route" in routes[0] ? routes.map((r) => r.route) : routes;
  const index = normalized.findIndex(
    (route2) => route2.pattern.test({ pathname: getRouteMatchingPathname(url) })
  );
  const route = normalized[index];
  const match = execRouteMatch(url, route);
  return route && match ? { index, route, match } : void 0;
}
function execRouteMatch(url, route) {
  var _a;
  return (_a = route == null ? void 0 : route.pattern.exec({ pathname: getRouteMatchingPathname(url) })) == null ? void 0 : _a.pathname;
}
var htmlExtRE2 = /\.html$/;
function getRouteMatchingPathname(url) {
  return url.pathname.replace(htmlExtRE2, "");
}
function normalizeURL(url) {
  url.pathname = url.pathname.replace(/\/index.html$/, "/");
  return url;
}

// src/shared/utils/string.ts
var wordSeparators = /[\s\-_]+/;
function toPascalCase(str) {
  const words = str.split(wordSeparators);
  const len = words.length;
  const mappedWords = new Array(len);
  for (let i = 0; i < len; i++) {
    const word = words[i];
    if (word === "")
      continue;
    mappedWords[i] = word[0].toUpperCase() + word.slice(1);
  }
  return mappedWords.join("");
}
function uppercaseFirstLetter(str) {
  return str.charAt(0).toUpperCase() + str.slice(1);
}
function lowercaseFirstLetter(str) {
  return str.charAt(0).toLowerCase() + str.slice(1);
}

// src/shared/utils/support.ts
var inBrowser = typeof window !== "undefined";

// src/node/app/alias.ts
var VM_PREFIX = ":virtual/vitebook";
var virtualModuleId = {
  noop: `${VM_PREFIX}/noop`,
  pages: `${VM_PREFIX}/pages`,
  layouts: `${VM_PREFIX}/layouts`,
  client: `${VM_PREFIX}/client`,
  app: `${VM_PREFIX}/app`
};
var virtualModuleRequestPath = Object.keys(virtualModuleId).reduce(
  (paths, key) => ({
    ...paths,
    [key]: `/${virtualModuleId[key]}`
  }),
  {}
);
var virtualAliases = Object.keys(virtualModuleId).reduce(
  (alias, key) => ({
    ...alias,
    [virtualModuleId[key]]: virtualModuleRequestPath[key]
  }),
  {}
);

// ../../node_modules/.pnpm/kleur@4.1.5/node_modules/kleur/index.mjs
var FORCE_COLOR;
var NODE_DISABLE_COLORS;
var NO_COLOR;
var TERM;
var isTTY = true;
if (typeof process !== "undefined") {
  ({ FORCE_COLOR, NODE_DISABLE_COLORS, NO_COLOR, TERM } = process.env || {});
  isTTY = process.stdout && process.stdout.isTTY;
}
var $ = {
  enabled: !NODE_DISABLE_COLORS && NO_COLOR == null && TERM !== "dumb" && (FORCE_COLOR != null && FORCE_COLOR !== "0" || isTTY),
  reset: init(0, 0),
  bold: init(1, 22),
  dim: init(2, 22),
  italic: init(3, 23),
  underline: init(4, 24),
  inverse: init(7, 27),
  hidden: init(8, 28),
  strikethrough: init(9, 29),
  black: init(30, 39),
  red: init(31, 39),
  green: init(32, 39),
  yellow: init(33, 39),
  blue: init(34, 39),
  magenta: init(35, 39),
  cyan: init(36, 39),
  white: init(37, 39),
  gray: init(90, 39),
  grey: init(90, 39),
  bgBlack: init(40, 49),
  bgRed: init(41, 49),
  bgGreen: init(42, 49),
  bgYellow: init(43, 49),
  bgBlue: init(44, 49),
  bgMagenta: init(45, 49),
  bgCyan: init(46, 49),
  bgWhite: init(47, 49)
};
function run(arr, str) {
  let i = 0, tmp, beg = "", end = "";
  for (; i < arr.length; i++) {
    tmp = arr[i];
    beg += tmp.open;
    end += tmp.close;
    if (!!~str.indexOf(tmp.close)) {
      str = str.replace(tmp.rgx, tmp.close + tmp.open);
    }
  }
  return beg + str + end;
}
function chain(has, keys) {
  let ctx = { has, keys };
  ctx.reset = $.reset.bind(ctx);
  ctx.bold = $.bold.bind(ctx);
  ctx.dim = $.dim.bind(ctx);
  ctx.italic = $.italic.bind(ctx);
  ctx.underline = $.underline.bind(ctx);
  ctx.inverse = $.inverse.bind(ctx);
  ctx.hidden = $.hidden.bind(ctx);
  ctx.strikethrough = $.strikethrough.bind(ctx);
  ctx.black = $.black.bind(ctx);
  ctx.red = $.red.bind(ctx);
  ctx.green = $.green.bind(ctx);
  ctx.yellow = $.yellow.bind(ctx);
  ctx.blue = $.blue.bind(ctx);
  ctx.magenta = $.magenta.bind(ctx);
  ctx.cyan = $.cyan.bind(ctx);
  ctx.white = $.white.bind(ctx);
  ctx.gray = $.gray.bind(ctx);
  ctx.grey = $.grey.bind(ctx);
  ctx.bgBlack = $.bgBlack.bind(ctx);
  ctx.bgRed = $.bgRed.bind(ctx);
  ctx.bgGreen = $.bgGreen.bind(ctx);
  ctx.bgYellow = $.bgYellow.bind(ctx);
  ctx.bgBlue = $.bgBlue.bind(ctx);
  ctx.bgMagenta = $.bgMagenta.bind(ctx);
  ctx.bgCyan = $.bgCyan.bind(ctx);
  ctx.bgWhite = $.bgWhite.bind(ctx);
  return ctx;
}
function init(open, close) {
  let blk = {
    open: `\x1B[${open}m`,
    close: `\x1B[${close}m`,
    rgx: new RegExp(`\\x1b\\[${close}m`, "g")
  };
  return function(txt) {
    if (this !== void 0 && this.has !== void 0) {
      !!~this.has.indexOf(open) || (this.has.push(open), this.keys.push(blk));
      return txt === void 0 ? this : $.enabled ? run(this.keys, txt + "") : txt + "";
    }
    return txt === void 0 ? chain([open], [blk]) : $.enabled ? run([blk], txt + "") : txt + "";
  };
}
var kleur_default = $;

// src/node/app/build/adapter/auto.ts
var adapters = [
  {
    name: "vercel",
    loader: () => import("./adapter.S2WMZE5V.js"),
    test: () => !!process.env.VERCEL
  }
];
function createAutoBuildAdapter(config) {
  const using = (name) => console.log(kleur_default.bold(kleur_default.magenta(`
\u{1F3D7}\uFE0F  Using ${name} build adapter`)));
  return async (...args) => {
    for (const adapter of adapters) {
      if (adapter.name === (config == null ? void 0 : config.use) || adapter.test()) {
        const { default: createAdapter } = await adapter.loader();
        using(adapter.name);
        return createAdapter(config == null ? void 0 : config[adapter.name])(...args);
      }
    }
    const staticAdapter = createStaticBuildAdapter();
    using(staticAdapter.name);
    return staticAdapter(...args);
  };
}

// src/node/app/build/adapter/BuildAdapterUtils.ts
import fsp from "fs/promises";
import fs6 from "node:fs";
import path4 from "node:path";
import ora2 from "ora";

// src/node/utils/crypto.ts
import { createHash } from "node:crypto";
function hash(content) {
  return createHash("sha1").update(content).digest("hex").substring(0, 8);
}

// src/node/utils/fs.ts
import { createHash as createHash2 } from "node:crypto";
import fs2 from "node:fs";
import path2 from "node:path";

// src/node/utils/path.ts
import fs from "node:fs";
import os from "node:os";
import path from "node:path";
function trimExt(filePath) {
  return filePath.substring(0, filePath.lastIndexOf(".")) || filePath;
}
var resolveRelativePath = (base, filePath) => {
  base = normalizePath(base);
  filePath = normalizePath(filePath);
  return path.posix.isAbsolute(filePath) ? filePath : path.posix.resolve(
    fs.lstatSync(base).isDirectory() ? base : path.dirname(base),
    filePath
  );
};
var isSubpath = (parent, filePath) => {
  const relative = path.posix.relative(parent, filePath);
  return !!relative && !relative.startsWith("..") && !path.isAbsolute(relative);
};
var isWindows = os.platform() === "win32";
function normalizePath(id) {
  return path.posix.normalize(isWindows ? id.replace(/\\/g, "/") : id);
}

// src/node/utils/fs.ts
var isTypeScriptFile = (filePath) => /\.(ts|tsx)($|\?)/.test(filePath);
var isCommonJsFile = (filePath) => /\.cjs($|\?)/.test(filePath);
async function readRecentlyChangedFile(file) {
  const content = fs2.readFileSync(file, "utf-8");
  if (!content) {
    const mtime = fs2.statSync(file).mtimeMs;
    await new Promise((r) => {
      let n = 0;
      const poll = async () => {
        n++;
        const newMtime = fs2.statSync(file).mtimeMs;
        if (newMtime !== mtime || n > 10) {
          r(0);
        } else {
          setTimeout(poll, 10);
        }
      };
      setTimeout(poll, 10);
    });
    return fs2.readFileSync(file, "utf-8");
  } else {
    return content;
  }
}
function checksumFile(algorithm, path13) {
  return new Promise((resolve, reject) => {
    const hash2 = createHash2(algorithm);
    const stream = fs2.createReadStream(path13);
    stream.on("error", (err) => reject(err));
    stream.on("data", (chunk) => hash2.update(chunk));
    stream.on("end", () => resolve(hash2.digest("hex")));
  });
}
async function ensureDir(dir) {
  if (fs2.existsSync(dir))
    return;
  await fs2.promises.mkdir(dir, { recursive: true });
}
async function ensureFile(filePath) {
  filePath = normalizePath(filePath);
  if (fs2.existsSync(filePath))
    return;
  await ensureDir(path2.posix.dirname(filePath));
  await fs2.promises.writeFile(filePath, "", { encoding: "utf-8" });
}
function copyDir(srcDir, destDir) {
  srcDir = normalizePath(srcDir);
  destDir = normalizePath(destDir);
  fs2.mkdirSync(destDir, { recursive: true });
  for (let file of fs2.readdirSync(srcDir)) {
    file = normalizePath(file);
    const srcFile = path2.posix.resolve(srcDir, file);
    const destFile = path2.posix.resolve(destDir, file);
    copyFile(srcFile, destFile);
  }
}
function copyFile(src, dest) {
  const stat = fs2.statSync(src);
  if (stat.isDirectory()) {
    copyDir(src, dest);
  } else {
    fs2.copyFileSync(src, dest);
  }
}
function mkdirp(dir) {
  try {
    fs2.mkdirSync(dir, { recursive: true });
  } catch (e) {
    if (e.code === "EEXIST")
      return;
    throw e;
  }
}
function rimraf(path13) {
  fs2.rmSync(path13, { force: true, recursive: true });
}

// src/node/utils/logger.ts
import ora from "ora";
var LoggerColor = Object.freeze({
  Info: kleur_default.cyan,
  Tip: kleur_default.magenta,
  Success: kleur_default.green,
  Warn: kleur_default.yellow,
  Error: kleur_default.red
});
var LoggerIcon = Object.freeze({
  Info: "\u2139\uFE0F",
  Tip: "\u{1F4A1}",
  Success: "\u2705",
  Warn: "\u26A0\uFE0F ",
  Error: "\u{1F6A8}"
});
var formatInfoTitle = (title) => LoggerColor.Info(`
${LoggerIcon.Info}  ${title}
`);
var formatTipTitle = (title) => LoggerColor.Tip(`
${LoggerIcon.Tip} ${title}
`);
var formatSuccessTitle = (title) => LoggerColor.Success(`
${LoggerIcon.Success} ${title}
`);
var formatWarnTitle = (title) => LoggerColor.Warn(`
${LoggerIcon.Warn} ${title}
`);
var formatErrorTitle = (title) => LoggerColor.Error(`
${LoggerIcon.Error} ${title}
`);
var createError = (message) => {
  return new Error(formatErrorTitle(message ?? ""));
};
var logger = {
  info: (title, ...args) => {
    console.info(formatInfoTitle(title), ...args);
  },
  tip: (title, ...args) => {
    console.info(formatTipTitle(title), ...args);
  },
  success: (title, ...args) => {
    console.info(formatSuccessTitle(title), ...args);
  },
  warn: (title, ...args) => {
    console.warn(formatWarnTitle(title), ...args);
  },
  error: (title, ...args) => {
    console.error(formatErrorTitle(title), ...args);
  },
  withSpinner: (pendingTitle, options = {}) => async (target) => {
    const spinner = ora();
    const successTitle = options.successTitle ?? pendingTitle;
    const errorTitle = options.errorTitle ?? pendingTitle;
    try {
      spinner.start(kleur_default.bold(`${pendingTitle}...`));
      const startTime = Date.now();
      const result = await target();
      const endTime = ((Date.now() - startTime) / 1e3).toFixed(2);
      spinner.stopAndPersist({
        symbol: "",
        text: formatSuccessTitle(
          options.timed ? `${successTitle} in ${kleur_default.underline(`${endTime}s`)}` : successTitle
        )
      });
      return result;
    } catch (e) {
      spinner.stopAndPersist({
        symbol: "",
        text: formatErrorTitle(errorTitle)
      });
      throw e;
    }
  }
};

// src/node/utils/module.ts
import { createRequire } from "module";
var esmRequire = () => createRequire(import.meta.url);

// src/node/utils/sort.ts
import LRUCache from "lru-cache";
var orderedPageTokenRE = /^\[(\d)\]/;
var sortCache = new LRUCache({ max: 2048 });
function comparePaths(pathA, pathB, { ordered = false } = {}) {
  const cacheKey = pathA + pathB;
  if (sortCache.has(cacheKey))
    return sortCache.get(cacheKey);
  const compare = () => {
    var _a, _b;
    const tokensA = slashedSplit(pathA);
    const tokensB = slashedSplit(pathB);
    const len = Math.max(tokensA.length, tokensB.length);
    for (let i = 0; i < len; i++) {
      if (!(i in tokensA)) {
        return -1;
      }
      if (!(i in tokensB)) {
        return 1;
      }
      const tokenA = tokensA[i].toLowerCase();
      const tokenB = tokensB[i].toLowerCase();
      if (ordered) {
        const tokenAOrderNo = (_a = tokensA[i].match(orderedPageTokenRE)) == null ? void 0 : _a[1];
        const tokenBOrderNo = (_b = tokensA[i].match(orderedPageTokenRE)) == null ? void 0 : _b[1];
        if (tokenAOrderNo && tokenBOrderNo) {
          const result2 = tokenAOrderNo < tokenBOrderNo ? -1 : 1;
          return result2;
        }
      }
      if (tokenA === tokenB) {
        continue;
      }
      const isTokenADir = tokenA[tokenA.length - 1] === "/";
      const isTokenBDir = tokenB[tokenB.length - 1] === "/";
      if (isTokenADir === isTokenBDir) {
        const result2 = tokenA < tokenB ? -1 : 1;
        return result2;
      } else {
        const result2 = isTokenADir ? 1 : -1;
        return result2;
      }
    }
    return 0;
  };
  const result = compare();
  sortCache.set(cacheKey, result);
  return result;
}

// src/node/utils/time.ts
function ms(val) {
  const s = 1e3;
  const m = s * 60;
  const msAbs = Math.abs(val);
  if (msAbs >= m) {
    return Math.round(val / m) + "m";
  }
  if (msAbs >= s) {
    return Math.round(val / s) + "s";
  }
  return Number(val.toFixed(2)) + "ms";
}

// src/node/app/create/app-dirs.ts
import fs3 from "node:fs";
import path3 from "node:path";
import { searchForWorkspaceRoot } from "vite";
function createAppDirectories(root, config) {
  const cwdDir = createDirectory(process.cwd());
  const rootDir = createDirectory(root);
  const workspaceDir = createDirectory(
    searchForWorkspaceRoot(cwdDir.path, rootDir.path)
  );
  const appDir = createDirectory(config.dirs.app);
  const buildDir = createDirectory(config.dirs.build);
  const tmpDir = createDirectory(rootDir.resolve(".vitebook"));
  const clientDir = createDirectory(rootDir.resolve(".vitebook/client"));
  const serverDir = createDirectory(rootDir.resolve(".vitebook/server"));
  const publicDir = createDirectory(config.dirs.public);
  return {
    cwd: cwdDir,
    workspace: workspaceDir,
    root: rootDir,
    app: appDir,
    tmp: tmpDir,
    build: buildDir,
    client: clientDir,
    server: serverDir,
    public: publicDir
  };
}
function createDirectory(dirname) {
  const resolve = (...args) => args.length === 1 && path3.posix.isAbsolute(normalizePath(args[0])) ? normalizePath(args[0]) : path3.posix.resolve(dirname, ...args.map(normalizePath));
  const relative = (...args) => path3.posix.relative(dirname, path3.posix.join(...args.map(normalizePath)));
  const read = (filePath) => fs3.readFileSync(resolve(filePath), "utf-8");
  const write = (filePath, data) => fs3.writeFileSync(resolve(filePath), data);
  return {
    path: dirname,
    resolve,
    relative,
    read,
    write
  };
}

// src/node/app/plugins/core/index-html.ts
import fs4 from "node:fs";
var DEFAULT_INDEX_HTML = `
<!DOCTYPE html>
<html lang="en-US">
  <head>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width,initial-scale=1" />
    <meta name="generator" content="vitebook@{{ version }}" />
    <!--@vitebook/head-->
  </head>
  <body>
    <noscript>You need to enable JavaScript to run this app.</noscript>
    <div id="app"><!--@vitebook/app--></div>
    <!--@vitebook/body-->
  </body>
</html>
`;
function readIndexHtmlFile(app, { dev = true } = {}) {
  const indexPath = app.dirs.app.resolve("index.html");
  let html = fs4.existsSync(indexPath) ? fs4.readFileSync(indexPath, "utf-8") : DEFAULT_INDEX_HTML;
  if (dev) {
    html = html.replace(
      "<!--@vitebook/head-->",
      '<script type="module" src="/:virtual/vitebook/client"><\/script>\n	<!--@vitebook/head-->'
    );
  }
  return html.replace("{{ version }}", app.version);
}

// src/node/app/build/chunks.ts
function extendManualChunks() {
  return (id) => {
    if (id.includes("vite/"))
      return "vite";
    if (id.includes("node_modules")) {
      if (/\/svelte\//.test(id))
        return "svelte";
      if (/@vitebook/.test(id))
        return "vitebook";
    }
    return null;
  };
}
function resolvePageChunks(app, page, { entryChunk, appChunk, viteManifest }, modules) {
  const imports = /* @__PURE__ */ new Set();
  const dynamicImports = /* @__PURE__ */ new Set();
  const assets = /* @__PURE__ */ new Set();
  const seen = /* @__PURE__ */ new WeakSet();
  const collectChunks2 = (chunk, isPage = false) => {
    if (!chunk || seen.has(chunk) || !isPage && chunk.file.includes("@page")) {
      return;
    }
    if (chunk.assets) {
      for (const id of chunk.assets) {
        const asset = viteManifest[id];
        if (asset)
          assets.add(asset.file);
      }
    }
    if (chunk.imports) {
      for (const id of chunk.imports) {
        const chunk2 = viteManifest[id];
        if (chunk2) {
          collectChunks2(chunk2);
          imports.add(chunk2.file);
        }
      }
    }
    if (chunk.dynamicImports) {
      for (const id of chunk.dynamicImports) {
        const chunk2 = viteManifest[id];
        if (chunk2 && !imports.has(chunk2.file) && !chunk2.file.includes("@page") && !chunk2.file.includes("@layout")) {
          dynamicImports.add(chunk2.file);
        }
      }
    }
    seen.add(chunk);
  };
  const entryId = app.dirs.root.relative(entryChunk.facadeModuleId);
  collectChunks2(viteManifest[entryId]);
  imports.add(entryChunk.fileName);
  const appId = app.dirs.root.relative(appChunk.facadeModuleId);
  collectChunks2(viteManifest[appId]);
  imports.add(appChunk.fileName);
  const layoutChunks = page.layouts.map(
    (index) => app.nodes.layouts.getByIndex(index)
  );
  for (const layout of layoutChunks) {
    const chunk = viteManifest[layout.rootPath];
    if (chunk) {
      collectChunks2(chunk);
      imports.add(chunk.file);
    }
  }
  if (modules) {
    for (const id of modules) {
      const chunk = viteManifest[id];
      if (chunk) {
        collectChunks2(viteManifest[id]);
        imports.add(chunk.file);
      }
    }
  }
  const pageChunk = viteManifest[page.rootPath];
  if (pageChunk) {
    collectChunks2(pageChunk, true);
    imports.add(pageChunk.file);
  }
  return {
    assets: Array.from(assets),
    imports: Array.from(imports),
    dynamicImports: Array.from(dynamicImports)
  };
}

// src/node/app/build/crawl.ts
var DOCTYPE = "DOCTYPE";
var CDATA_OPEN = "[CDATA[";
var CDATA_CLOSE = "]]>";
var COMMENT_OPEN = "--";
var COMMENT_CLOSE = "-->";
var TAG_OPEN = /[a-zA-Z]/;
var TAG_CHAR = /[a-zA-Z0-9]/;
var ATTRIBUTE_NAME = /[^\t\n\f />"'=]/;
var WHITESPACE = /[\s\n\r]/;
function crawl(html) {
  const hrefs = [];
  let i = 0;
  main:
    while (i < html.length) {
      const char = html[i];
      if (char === "<") {
        if (html[i + 1] === "!") {
          i += 2;
          if (html.slice(i, i + DOCTYPE.length).toUpperCase() === DOCTYPE) {
            i += DOCTYPE.length;
            while (i < html.length) {
              if (html[i++] === ">") {
                continue main;
              }
            }
          }
          if (html.slice(i, i + CDATA_OPEN.length) === CDATA_OPEN) {
            i += CDATA_OPEN.length;
            while (i < html.length) {
              if (html.slice(i, i + CDATA_CLOSE.length) === CDATA_CLOSE) {
                i += CDATA_CLOSE.length;
                continue main;
              }
              i += 1;
            }
          }
          if (html.slice(i, i + COMMENT_OPEN.length) === COMMENT_OPEN) {
            i += COMMENT_OPEN.length;
            while (i < html.length) {
              if (html.slice(i, i + COMMENT_CLOSE.length) === COMMENT_CLOSE) {
                i += COMMENT_CLOSE.length;
                continue main;
              }
              i += 1;
            }
          }
        }
        const start = ++i;
        if (TAG_OPEN.test(html[start])) {
          while (i < html.length) {
            if (!TAG_CHAR.test(html[i])) {
              break;
            }
            i += 1;
          }
          const tag = html.slice(start, i).toUpperCase();
          if (tag === "SCRIPT" || tag === "STYLE") {
            while (i < html.length) {
              if (html[i] === "<" && html[i + 1] === "/" && html.slice(i + 2, i + 2 + tag.length).toUpperCase() === tag) {
                continue main;
              }
              i += 1;
            }
          }
          let href = "";
          const rel = "";
          while (i < html.length) {
            const start2 = i;
            const char2 = html[start2];
            if (char2 === ">")
              break;
            if (ATTRIBUTE_NAME.test(char2)) {
              i += 1;
              while (i < html.length) {
                if (!ATTRIBUTE_NAME.test(html[i])) {
                  break;
                }
                i += 1;
              }
              const name = html.slice(start2, i).toLowerCase();
              while (WHITESPACE.test(html[i]))
                i += 1;
              if (html[i] === "=") {
                i += 1;
                while (WHITESPACE.test(html[i]))
                  i += 1;
                let value;
                if (html[i] === "'" || html[i] === '"') {
                  const quote = html[i++];
                  const start3 = i;
                  let escaped = false;
                  while (i < html.length) {
                    if (!escaped) {
                      const char3 = html[i];
                      if (html[i] === quote) {
                        break;
                      }
                      if (char3 === "\\") {
                        escaped = true;
                      }
                    }
                    i += 1;
                  }
                  value = html.slice(start3, i);
                } else {
                  const start3 = i;
                  while (html[i] !== ">" && !WHITESPACE.test(html[i]))
                    i += 1;
                  value = html.slice(start3, i);
                  i -= 1;
                }
                if (name === "href") {
                  href = value;
                }
              } else {
                i -= 1;
              }
            }
            i += 1;
          }
          if (href && !/\bexternal\b/i.test(rel)) {
            hrefs.push(href);
          }
        }
      }
      i += 1;
    }
  return hrefs;
}

// src/node/app/build/log.ts
function logBadLinks(badLinks) {
  if (badLinks.size === 0)
    return;
  const logs = [
    "",
    `${LoggerIcon.Warn} ${kleur_default.bold(kleur_default.underline("BAD LINKS"))}`,
    ""
  ];
  for (const [pathname, { page, reason }] of badLinks) {
    logs.push(`- ${kleur_default.bold(pathname)}`);
    logs.push(`  - Reason: ${reason}`);
    if (page)
      logs.push(`  - Location: ${page.rootPath}`);
  }
  console.log(logs.join("\n"));
}
function logRoutesList({ level, ...build3 }) {
  const logs = [];
  if (level === "info") {
    logs.push("", `\u{1F4C4} ${kleur_default.bold(kleur_default.underline("PAGES"))}`, "");
    for (const link of build3.links.keys()) {
      const page = build3.links.get(link);
      const route = page.route.pathname.replace("{/}?{index}?{.html}?", "").slice(1);
      const pathname = link.slice(1, -1);
      const pattern = pathname !== route ? kleur_default.dim(` (${route})`) : "";
      logs.push(
        `- ${kleur_default.cyan(link.length === 1 ? "/" : pathname)}${pattern}`
      );
    }
  }
  if (level === "info") {
    logs.push("", `\u2699\uFE0F  ${kleur_default.bold(kleur_default.underline("ENDPOINTS"))}`, "");
    for (const link of build3.endpoints.keys()) {
      logs.push(`- ${kleur_default.cyan(link)}`);
    }
  }
  if (/(info|warn)/.test(level) && build3.redirects.size > 0) {
    logs.push("", `\u27A1\uFE0F  ${kleur_default.bold(kleur_default.underline("REDIRECTS"))}`, "");
    for (const link of build3.redirects.keys()) {
      const redirect = build3.redirects.get(link);
      logs.push(
        `- ${kleur_default.yellow(link)} -> ${kleur_default.yellow(redirect.to)} (${redirect.statusCode})`
      );
    }
  }
  if (/(info|warn|error)/.test(level) && build3.badLinks.size > 0) {
    logs.push("", `\u{1F6D1} ${kleur_default.bold(kleur_default.underline("NOT FOUND"))}`, "");
    for (const link of build3.badLinks.keys()) {
      logs.push(`- ${kleur_default.red(link)} (404)`);
    }
  }
  if (logs.length > 0) {
    console.log(logs.join("\n"));
    console.log();
  }
}
function logRoutesTree({ level, ...build3 }) {
  const newDir = (name) => ({
    name,
    path: []
  });
  const tree = newDir(".");
  const warnOnly = level === "warn";
  const errorOnly = level === "error";
  const redirectLinks = new Set(build3.redirects.keys());
  const filteredLinks = errorOnly ? build3.badLinks.keys() : warnOnly ? /* @__PURE__ */ new Set([...build3.badLinks.keys(), ...redirectLinks]) : /* @__PURE__ */ new Set([
    ...build3.badLinks.keys(),
    ...redirectLinks,
    ...build3.links.keys(),
    ...build3.endpoints.keys()
  ]);
  for (const link of filteredLinks) {
    const segments = noslash(link).split("/");
    let current = tree;
    for (const segment of segments.slice(0, -1)) {
      let nextDir = current.path.find((dir) => dir.name === segment);
      if (!nextDir) {
        nextDir = newDir(segment);
        current.path.push(nextDir);
      }
      current = nextDir;
    }
    if (build3.badLinks.has(link)) {
      current.badLink = true;
    } else if (build3.redirects.has(link)) {
      const redirect = build3.redirects.get(link);
      current.redirect = {
        path: redirect.to.slice(1, -1),
        statusCode: redirect.statusCode
      };
    } else {
      const route = build3.endpoints.has(link) ? build3.endpoints.get(link).route : build3.links.get(link).route;
      const pathname = route.pathname.replace("{/}?{index}?{.html}?", "").replace("{/}?", "");
      current.link = kleur_default.dim(` (${pathname.slice(1)})`);
    }
  }
  const PRINT_SYMBOLS = {
    BRANCH: "\u251C\u2500\u2500 ",
    EMPTY: "",
    INDENT: "    ",
    LAST_BRANCH: "\u2514\u2500\u2500 ",
    VERTICAL: "\u2502   "
  };
  const print = (tree2, depth, precedingSymbols) => {
    const lines = [];
    for (const [index, dir] of tree2.path.entries()) {
      const line = [precedingSymbols];
      const isLast = index === tree2.path.length - 1 && dir.path.length === 0;
      const name = dir.badLink ? `${kleur_default.red(dir.name)} ${kleur_default.red(kleur_default.bold("(404)"))}` : dir.redirect ? kleur_default.yellow(
        `${dir.name} -> ${dir.redirect.path} (${dir.redirect.statusCode})`
      ) : `${kleur_default[dir.link ? "cyan" : "white"](dir.name)}${kleur_default.dim(
        dir.link ?? ""
      )}`;
      line.push(isLast ? PRINT_SYMBOLS.LAST_BRANCH : PRINT_SYMBOLS.BRANCH);
      line.push(kleur_default.bold(`${dir.icon ? `${dir.icon} ` : ""}${name}`));
      lines.push(line.join(""));
      const dirLines = print(
        dir,
        depth + 1,
        precedingSymbols + (depth >= 1 ? isLast ? PRINT_SYMBOLS.INDENT : PRINT_SYMBOLS.VERTICAL : PRINT_SYMBOLS.EMPTY)
      );
      lines.push(...dirLines);
    }
    return lines;
  };
  if (tree.path.length > 0) {
    if (level === "info") {
      console.log(`
\u{1F5FA}\uFE0F  ${kleur_default.bold(kleur_default.underline("ROUTES"))}`, "");
    }
    console.log(kleur_default.bold(kleur_default.cyan(".")));
    console.log(print(tree, 1, "").join("\n"));
    console.log();
  }
}

// src/node/app/build/sitemap.ts
import { createFilter } from "@rollup/pluginutils";
import fs5 from "node:fs";
async function buildSitemap(app, links, config) {
  const baseUrl = config.baseUrl;
  if (!baseUrl) {
    app.logger.warn("Sitemap requires a base URL to build links");
    return null;
  }
  const filter = createFilter(config.include, config.exclude);
  const changefreq = isFunction(config.changefreq) ? config.changefreq : () => config.changefreq;
  const priority = isFunction(config.priority) ? config.priority : () => config.priority;
  const lastmodCache = /* @__PURE__ */ new Map();
  const lastmod = async (pathname) => {
    if (lastmodCache.has(pathname))
      return lastmodCache.get(pathname);
    const filePath = links.get(pathname).filePath;
    const mtime = (await fs5.promises.stat(filePath)).mtime;
    const date = mtime.toISOString().split("T")[0];
    lastmodCache.set(pathname, date);
    return date;
  };
  const urls = [
    ...await Promise.all(
      Array.from(links.keys()).filter(filter).map(async (pathname) => ({
        path: pathname,
        lastmod: await lastmod(pathname),
        changefreq: await changefreq(new URL(pathname, baseUrl)),
        priority: await priority(new URL(pathname, baseUrl))
      })).map(async (url) => buildSitemapURL(await url, baseUrl))
    ),
    ...config.entries.map((url) => buildSitemapURL(url, baseUrl))
  ].join("\n  ");
  const content = `<?xml version="1.0" encoding="UTF-8" ?>
<urlset
  xmlns="https://www.sitemaps.org/schemas/sitemap/0.9"
  xmlns:xhtml="https://www.w3.org/1999/xhtml"
  xmlns:mobile="https://www.google.com/schemas/sitemap-mobile/1.0"
  xmlns:news="https://www.google.com/schemas/sitemap-news/0.9"
  xmlns:image="https://www.google.com/schemas/sitemap-image/1.1"
  xmlns:video="https://www.google.com/schemas/sitemap-video/1.1"
>
  ${urls}
</urlset>`;
  return [config.filename, content];
}
function buildSitemapURL(url, baseUrl = "/") {
  const lastmod = url.lastmod ? `
    <lastmod>${url.lastmod ?? ""}</lastmod>` : "";
  return `<url>
    <loc>${baseUrl}${slash(url.path)}</loc>${lastmod}
    <changefreq>${url.changefreq ?? "weekly"}</changefreq>
    <priority>${url.priority ?? 0.7}</priority>
  </url>`;
}

// src/node/app/build/adapter/BuildAdapterUtils.ts
function getBuildAdapterUtils(app, bundles, build3) {
  const baseURL = app.vite.resolved.base;
  return {
    logger,
    icons: {
      info: LoggerIcon.Info,
      tip: LoggerIcon.Tip,
      success: LoggerIcon.Success,
      warn: LoggerIcon.Warn,
      error: LoggerIcon.Error
    },
    color: kleur_default,
    crawl,
    hash,
    normalizePath,
    trimExt,
    normalizeURL,
    mkdirp,
    rimraf,
    copyFile,
    copyDir,
    ensureDir,
    ensureFile,
    noslash,
    slash,
    endslash,
    noendslash,
    readFile: (filePath) => fsp.readFile(filePath, "utf-8"),
    writeFile: (filePath, fileContent) => fsp.writeFile(filePath, fileContent, "utf-8"),
    createFilesArray: () => [],
    writeFiles,
    isLinkExternal: (path13) => isLinkExternal(path13, baseURL),
    pluralize: (word, count) => count === 1 ? word : `${word}s`,
    createSpinner: ora2,
    resolveHTMLFilename: (url) => {
      const decodedRoute = decodeURI(isString(url) ? url : url.pathname);
      const filePath = decodedRoute.endsWith("/") ? `${decodedRoute}index.html` : decodedRoute;
      return noslash(filePath);
    },
    resolveDataFilename: (name) => `${DATA_ASSET_BASE_PATH}/${name}.json`.slice(1),
    resolvePageChunks: (page, modules) => resolvePageChunks(app, page, bundles.client, modules),
    buildSitemaps: async () => {
      if (app.config.sitemap.length === 0)
        return [];
      const sitemaps = app.config.sitemap.map((config) => buildSitemap(app, build3.links, config)).filter(Boolean);
      return await Promise.all(sitemaps);
    },
    createDataScriptTag: (dataAssetIds) => {
      const table = {};
      for (const id of dataAssetIds) {
        const data = build3.data.get(id);
        if (data && Object.keys(data.data).length > 0) {
          table[data.contentHash] = data.data;
        }
      }
      return [
        '<script id="__VBK_DATA__" type="application/json">',
        JSON.stringify(table),
        "<\/script>"
      ].join("");
    },
    createLinkTag: (rel, pathname) => createLinkTag(app, rel, pathname),
    createPreloadTag: (pathname) => createPreloadTag(app, pathname),
    createRedirectMetaTag: (url) => `<meta http-equiv="refresh" content="${escapeHTML(`0;url=${url}`)}">`,
    guessPackageManager: () => guessPackageManager(app),
    findPreviewScriptName: () => findPreviewScriptName(app),
    logBadLinks: () => logBadLinks(build3.badLinks),
    logRoutes: ({ level, style: logStyle } = {}) => {
      const style = logStyle ?? app.config.routes.log;
      if (style !== "none") {
        const logger2 = style === "list" ? logRoutesList : style === "tree" ? logRoutesTree : style;
        logger2({
          level: level ?? app.config.routes.logLevel,
          ...build3
        });
      }
    },
    escapeHTML,
    createDirectory,
    getHTMLTemplate: () => readIndexHtmlFile(app, { dev: false })
  };
}
function createLinkTag(app, rel, fileName) {
  if (!fileName)
    return "";
  const baseUrl = noendslash(app.vite.resolved.base);
  const href = `${baseUrl}${slash(fileName)}`;
  return `<link rel="${rel}" href="${href}" />`;
}
function createPreloadTag(app, fileName) {
  if (!fileName)
    return "";
  const baseUrl = noendslash(app.vite.resolved.base);
  const href = `${baseUrl}${slash(fileName)}`;
  if (fileName.endsWith(".js")) {
    return `<link rel="modulepreload" crossorigin href="${href}" />`;
  } else if (fileName.endsWith(".css")) {
    return `<link rel="stylesheet" href="${href}" />`;
  } else if (fileName.endsWith(".woff")) {
    return ` <link rel="preload" href="${href}" as="font" type="font/woff" crossorigin />`;
  } else if (fileName.endsWith(".woff2")) {
    return ` <link rel="preload" href="${href}" as="font" type="font/woff2" crossorigin />`;
  } else if (fileName.endsWith(".gif")) {
    return ` <link rel="preload" href="${href}" as="image" type="image/gif" />`;
  } else if (fileName.endsWith(".jpg") || fileName.endsWith(".jpeg")) {
    return ` <link rel="preload" href="${href}" as="image" type="image/jpeg" />`;
  } else if (fileName.endsWith(".png")) {
    return ` <link rel="preload" href="${href}" as="image" type="image/png" />`;
  }
  return "";
}
function guessPackageManager(app) {
  if (fs6.existsSync(app.dirs.root.resolve("pnpm-lock.yaml"))) {
    return "pnpm";
  }
  if (fs6.existsSync(app.dirs.root.resolve("yarn.lock"))) {
    return "yarn";
  }
  return "npm";
}
async function findPreviewScriptName(app) {
  try {
    const packageJson = app.dirs.root.resolve("package.json");
    if (fs6.existsSync(packageJson)) {
      const content = fs6.readFileSync(packageJson, "utf-8");
      const json = JSON.parse(content);
      const script = Object.keys(json.scripts ?? {}).find((script2) => {
        return json.scripts[script2].includes("vite preview");
      });
      return script;
    }
  } catch (e) {
  }
  return null;
}
async function writeFiles(files, resolveFilePath, resolvePendingMessage, resolveSuccessMessage) {
  const writingSpinner = ora2();
  const filesCount = kleur_default.underline(files.length);
  const pendingMessage = resolvePendingMessage == null ? void 0 : resolvePendingMessage(filesCount);
  const successMessage = resolveSuccessMessage == null ? void 0 : resolveSuccessMessage(filesCount);
  writingSpinner.start(kleur_default.bold(pendingMessage));
  await Promise.all(
    files.map(async ({ filename, content }) => {
      const filePath = normalizePath(resolveFilePath(filename));
      await ensureDir(path4.dirname(filePath));
      await fsp.writeFile(filePath, content);
    })
  );
  writingSpinner.stopAndPersist({
    text: kleur_default.bold(successMessage),
    symbol: LoggerIcon.Success
  });
}

// src/node/app/config/resolve-config.ts
import path5 from "node:path";
function resolveAppConfig(root, {
  build: build3 = {},
  dirs = {},
  isDebug = !!process.env.DEBUG,
  client = {},
  routes = {},
  markdown = {},
  sitemap
}) {
  const _cwd = normalizePath(process.cwd());
  const _root = resolveRelativePath(_cwd, root);
  const _app = resolveRelativePath(_root, dirs.app ?? "app");
  const _public = resolveRelativePath(_app, dirs.public ?? "public");
  const _output = resolveRelativePath(_root, dirs.build ?? "build");
  const __build = {
    adapter: build3.adapter ?? createAutoBuildAdapter()
  };
  const __client = {
    app: client.app ? path5.posix.relative(_root, normalizePath(client.app)) : "",
    configFiles: client.configFiles ?? []
  };
  const pageExts = `md,svelte,vue,jsx,tsx`;
  const endpointExts = "js,ts";
  const __routes = {
    entries: routes.entries ?? [],
    matchers: [
      ...routes.matchers ?? [],
      { name: "int", matcher: /\d+/ },
      { name: "str", matcher: /\w+/ },
      { name: "bool", matcher: /(true|false)/ },
      (route) => route.replace(/\[\.\.\.(.*?)\]/g, ":$1*"),
      (route) => route.replace(/\[(.*?)\]/g, ":$1")
    ],
    log: routes.log ?? "tree",
    logLevel: routes.logLevel ?? "warn",
    pages: {
      include: [`**/@404.{${pageExts}}`, `**/*@page*.{${pageExts}}`],
      exclude: [],
      ...routes.pages
    },
    layouts: {
      include: [
        `**/*@layout.{${pageExts}}`,
        `**/*@layout.reset.{${pageExts}}`,
        `**/@layouts/**/*.{${pageExts}}`,
        `**/@layouts/**/*.reset.{${pageExts}}`
      ],
      exclude: [],
      ...routes.layouts
    },
    endpoints: {
      include: [`**/@http.{${endpointExts}}`],
      exclude: [],
      ...routes.endpoints
    }
  };
  const __markdown = {
    include: markdown.include ?? /\.md($|\?)/,
    exclude: markdown.exclude ?? [],
    markdoc: markdown.markdoc ?? {},
    highlighter: false,
    shiki: { theme: "material-palenight", langs: [] },
    hastToHtml: {},
    nodes: {
      include: [`**/@markdoc/**/*.{${pageExts}}`],
      exclude: [],
      ...markdown.nodes
    },
    transformAst: [],
    transformContent: [],
    transformMeta: [],
    transformOutput: [],
    transformTreeNode: [],
    ...markdown
  };
  const __sitemapConfig = {
    baseUrl: null,
    filename: "sitemap.xml",
    changefreq: "weekly",
    priority: 0.7,
    include: /.*/,
    exclude: null,
    entries: []
  };
  const __sitemap = (isArray(sitemap) ? sitemap : [sitemap]).filter(Boolean).map((config) => ({ ...__sitemapConfig, ...config }));
  return {
    build: __build,
    dirs: {
      app: _app,
      build: _output,
      public: _public
    },
    client: __client,
    routes: __routes,
    markdown: __markdown,
    sitemap: __sitemap,
    isBuild: false,
    isSSR: false,
    isDebug
  };
}

// src/node/app/markdoc/MarkdocSchema.ts
import Markdoc from "@markdoc/markdoc";
var MarkdocSchema = class {
  constructor() {
    this.hmrFiles = /* @__PURE__ */ new Map();
  }
  init(app) {
    this._app = app;
    this._nodes = app.nodes.markdoc;
  }
  getOwnedConfig(ownerFilePath) {
    var _a, _b;
    const base = this._app.config.markdown.markdoc;
    const isPage = this._app.nodes.pages.is(ownerFilePath);
    const nodes = isPage ? this._getOwnedNodesConfig(ownerFilePath) : {};
    const tags = isPage ? this._getOwnedTagsConfig(ownerFilePath) : {};
    const config = {
      ...base,
      variables: {
        ...base.variables,
        file: {
          ...(_a = base.variables) == null ? void 0 : _a.file,
          path: ownerFilePath
        },
        vite: {
          ...(_b = base.variables) == null ? void 0 : _b.vite,
          mode: this._app.vite.resolved.mode,
          env: this._app.vite.resolved.env
        }
      },
      nodes: {
        ...base.nodes,
        ...nodes
      },
      tags: {
        import: {
          render: void 0,
          selfClosing: true,
          transform(node) {
            return new Markdoc.Tag("import", node.attributes);
          }
        },
        ...base.tags,
        ...tags
      }
    };
    return config;
  }
  resolveOwnedImports(ownerFilePath) {
    return this._nodes.getOwnedNodes(ownerFilePath, "*").map((node) => {
      const rootPath = this._app.dirs.root.relative(node.filePath);
      return `import ${node.cname} from "/${rootPath}";`;
    }).filter(Boolean);
  }
  _getOwnedNodesConfig(ownerFilePath) {
    const nodes = {};
    for (const node of this._nodes.getOwnedNodes(ownerFilePath, "node")) {
      nodes[node.name] = {
        render: node.cname,
        transform: (_node, config) => {
          return new Markdoc.Tag(
            node.cname,
            _node.attributes,
            _node.transformChildren(config)
          );
        }
      };
      this._markFileForHMR(node.filePath, ownerFilePath);
    }
    return nodes;
  }
  _getOwnedTagsConfig(ownerFilePath) {
    const tags = {};
    for (const tag of this._nodes.getOwnedNodes(ownerFilePath, "tag")) {
      tags[tag.name] = {
        render: tag.cname,
        selfClosing: tag.inline,
        transform: (node, config) => {
          return new Markdoc.Tag(
            tag.cname,
            node.attributes,
            node.transformChildren(config)
          );
        }
      };
      this._markFileForHMR(tag.filePath, ownerFilePath);
    }
    return tags;
  }
  _markFileForHMR(nodeFilePath, ownerFilePath) {
    this.hmrFiles.set(
      nodeFilePath,
      (this.hmrFiles.get(nodeFilePath) ?? /* @__PURE__ */ new Set()).add(ownerFilePath)
    );
  }
};

// src/node/app/markdoc/parse-markdown.ts
import Markdoc2 from "@markdoc/markdoc";
import matter from "gray-matter";
import yaml from "js-yaml";
import LRUCache2 from "lru-cache";
import fs9 from "node:fs";
import path10 from "node:path";

// src/node/app/nodes/FileNodes.ts
import { createFilter as createFilter2 } from "@rollup/pluginutils";
import { globbySync } from "globby";

// src/node/app/nodes/resolve-route.ts
import path6 from "node:path";
var PAGE_ORDER_RE = /^\[(\d*)\]/;
function stripRouteMetaFromFilePath(filePath) {
  const ext = path6.posix.extname(filePath);
  const stripped = filePath.replace(
    new RegExp(`@(\\w|\\+)+\\.${ext.slice(1)}$`, "i"),
    ext
  );
  return stripped === ext ? `index${ext}` : ext ? stripped.replace(`/${ext}`, `/index${ext}`) : stripped.replace(/\/?$/, "/index.html");
}
function stripRouteInfoFromFilePath(filePath) {
  return stripRouteMetaFromFilePath(stripPageOrderFromPath(filePath));
}
function normalizeTransformMatcher(value) {
  if (value instanceof RegExp) {
    const regexStr = value.toString();
    value = regexStr.startsWith("/(") ? regexStr.slice(1, -1) : `(${regexStr.slice(1, -1)})`;
  }
  return value ?? "";
}
function resolveRouteFromFilePath(routesDir, filePath, matchers = []) {
  var _a;
  filePath = normalizePath(filePath);
  const routePath = path6.posix.relative(routesDir, filePath);
  const basename = path6.posix.basename(routePath);
  const orderMatch = (_a = basename.match(PAGE_ORDER_RE)) == null ? void 0 : _a[1];
  const order = orderMatch ? Number(orderMatch) : void 0;
  const isNotFound = basename.includes("@404");
  const isEndpoint = basename.includes("@http");
  let route = stripRouteInfoFromFilePath(routePath);
  for (const matcher of matchers) {
    if (isFunction(matcher)) {
      const result = matcher(route, { filePath });
      if (result)
        route = result;
    } else {
      route = route.replace(
        `[${matcher.name}]`,
        normalizeTransformMatcher(matcher.matcher)
      );
    }
  }
  const resolveStaticPath = () => {
    const url = new URL(route.toLowerCase(), "http://v/");
    return url.pathname.replace(/\..+($|\\?)/i, "{.html}?").replace(/\/(README|index){.html}\?($|\?)/i, "{/}?{index}?{.html}?");
  };
  const dynamic = isNotFound || isRoutePathDynamic(route);
  const pathname = isNotFound ? `${route.replace(path6.posix.basename(route), "")}(.*?)` : dynamic || isEndpoint ? slash(
    trimExt(route).replace(
      /\/(index)?(\.html)?$/,
      isEndpoint ? "{/}?" : "{/}?{index}?{.html}?"
    )
  ) : resolveStaticPath();
  const score = calcRoutePathScore(pathname);
  const pattern = new URLPattern({ pathname });
  return {
    pattern,
    dynamic,
    pathname,
    order,
    score
  };
}
function resolveStaticRouteFromFilePath(routesDir, filePath) {
  const routePath = endslash(path6.posix.relative(routesDir, filePath));
  const url = new URL(
    stripRouteInfoFromFilePath(routePath).toLowerCase(),
    "http://localhost"
  );
  return url.pathname.replace(/\..+($|\\?)/i, ".html").replace(/\/(README|index).html($|\\?)/i, "/");
}

// src/node/app/nodes/FileNodes.ts
var HAS_LOADER_RE = /(export function loader|export async function loader|export const loader)/;
var FileNodes = class {
  constructor() {
    this._nodes = [];
  }
  get size() {
    return this._nodes.length;
  }
  async init(app, options) {
    this._app = app;
    this._options = options;
    this._filter = createFilter2(options.include, options.exclude);
    await this._discover();
    app.disposal.add();
  }
  async _discover() {
    const filePaths = this._getFilePaths();
    await Promise.all(filePaths.map(this.add.bind(this)));
  }
  _getRootPath(filePath) {
    return this._app.dirs.root.relative(filePath);
  }
  _getFilePaths() {
    return globbySync(this._options.include, {
      absolute: true,
      cwd: this._app.dirs.app.path
    }).map(normalizePath).filter(this._filter);
  }
  remove(filePath) {
    var _a, _b;
    filePath = this.normalizePath(filePath);
    if (!this.has(filePath))
      return -1;
    const index = this.findIndex(filePath);
    this._nodes.splice(index, 1);
    (_b = (_a = this._options).onRemove) == null ? void 0 : _b.call(_a, index);
    return index;
  }
  getByIndex(index) {
    return this._nodes[index];
  }
  find(filePath) {
    filePath = this.normalizePath(filePath);
    return this._nodes.find((node) => node.filePath === filePath);
  }
  findIndex(filePath) {
    const node = this.find(filePath);
    return this._nodes.findIndex((n) => n === node);
  }
  has(filePath) {
    return !!this.find(filePath);
  }
  clear() {
    this._nodes = [];
  }
  is(filePath) {
    filePath = this.normalizePath(filePath);
    return this.has(filePath) || filePath.startsWith(this._app.dirs.app.path) && this._filter(filePath);
  }
  hasLoader(fileContent) {
    return HAS_LOADER_RE.test(fileContent);
  }
  normalizePath(filePath) {
    return normalizePath(filePath);
  }
  resolveRoute(filePath) {
    return resolveRouteFromFilePath(
      this._app.dirs.app.path,
      this.normalizePath(filePath),
      this._app.config.routes.matchers
    );
  }
  toArray() {
    return this._nodes;
  }
  [Symbol.iterator]() {
    let index = 0;
    return {
      next: () => {
        if (index < this._nodes.length) {
          return { value: this._nodes[index++], done: false };
        } else {
          return { done: true };
        }
      }
    };
  }
};

// src/node/app/nodes/EndpointNodes.ts
var EndpointNodes = class extends FileNodes {
  init(app, options) {
    return super.init(app, {
      include: app.config.routes.endpoints.include,
      exclude: app.config.routes.endpoints.exclude,
      ...options
    });
  }
  async add(filePath) {
    var _a, _b;
    const endpoint = {
      filePath,
      rootPath: this._getRootPath(filePath),
      route: this.resolveRoute(filePath)
    };
    this._nodes.push(endpoint);
    this._nodes = this._nodes.sort((a, b) => compareRoutes(a.route, b.route));
    (_b = (_a = this._options).onAdd) == null ? void 0 : _b.call(_a, endpoint);
    return endpoint;
  }
};

// src/node/app/nodes/LayoutNodes.ts
import fs7 from "node:fs";
import path7 from "node:path";
var STRIP_LAYOUTS_PATH = /\/@_nodes\/.+/;
var LAYOUT_NAME_RE = /(.*?)@layout/;
var LayoutNodes = class extends FileNodes {
  init(app, options) {
    return super.init(app, {
      include: app.config.routes.layouts.include,
      exclude: app.config.routes.layouts.exclude,
      ...options
    });
  }
  async add(filePath) {
    var _a, _b;
    filePath = this.normalizePath(filePath);
    const name = await this._getName(filePath);
    const rootPath = this._getRootPath(filePath);
    const owningDir = path7.posix.dirname(
      rootPath.replace(STRIP_LAYOUTS_PATH, "/root.md")
    );
    const fileContent = fs7.readFileSync(filePath, { encoding: "utf-8" });
    const hasLoader = this.hasLoader(fileContent);
    const reset = rootPath.includes(`.reset${path7.posix.extname(rootPath)}`);
    const layout = {
      id: `/${rootPath}`,
      name,
      filePath,
      rootPath,
      owningDir,
      hasLoader,
      reset
    };
    this._nodes.push(layout);
    this._nodes = this._nodes.sort((a, b) => {
      const segmentsA = a.rootPath.split(/\//g);
      const segmentsB = b.rootPath.split(/\//g);
      return segmentsA.length !== segmentsB.length ? segmentsA.length - segmentsB.length : path7.posix.basename(a.rootPath, path7.posix.extname(a.rootPath)) === "@layout" ? -1 : 1;
    });
    (_b = (_a = this._options).onAdd) == null ? void 0 : _b.call(_a, layout);
    return layout;
  }
  isOwnedBy(layout, ownerFilePath, layoutName) {
    const rootPath = this._getRootPath(ownerFilePath);
    const _layout = isString(layout) ? this.find(layout) : layout;
    return _layout && rootPath.startsWith(_layout.owningDir) && (_layout.name === "@layout" || _layout.name === layoutName);
  }
  getOwnedLayoutIndicies(ownerFilePath, layoutName) {
    let indicies = [];
    for (let i = 0; i < this._nodes.length; i++) {
      const layout = this._nodes[i];
      if (this.isOwnedBy(layout.filePath, ownerFilePath, layoutName)) {
        if (layout.reset)
          indicies = [];
        indicies.push(i);
      }
    }
    return indicies;
  }
  _getName(filePath) {
    var _a;
    const filename = path7.posix.basename(filePath, path7.posix.extname(filePath)).replace(/\.reset($|\/)/, "");
    const match = (_a = filename.match(LAYOUT_NAME_RE)) == null ? void 0 : _a[1];
    return match && match.length > 0 ? match : filename;
  }
};

// src/node/app/nodes/MarkdocNodes.ts
import path8 from "node:path";
var STRIP_MARKDOC_DIR_RE = /\/@markdoc\/.+/;
var MarkdocNodes = class extends FileNodes {
  init(app, options) {
    return super.init(app, {
      include: app.config.markdown.nodes.include,
      exclude: app.config.markdown.nodes.exclude,
      ...options
    });
  }
  async add(filePath) {
    var _a, _b;
    filePath = this.normalizePath(filePath);
    const rootPath = this._getRootPath(filePath);
    const routePath = this._app.dirs.app.relative(filePath);
    const name = path8.posix.basename(routePath, path8.posix.extname(routePath)).replace("@node", "").replace("@inline", "");
    const type = this.isNode(filePath) ? "node" : "tag";
    if (type === "node" && !isValidMarkdownNodeName(name)) {
      const validValues = `${kleur_default.bold("Valid values")}: ${Array.from(
        getValidNodeNames()
      )}`;
      this._app.logger.warn(
        `Invalid markdown node name [${kleur_default.bold(name)}]. 

${validValues}`
      );
    }
    const cname = toPascalCase(name);
    const inline = /@inline/.test(routePath);
    const owningDir = path8.posix.dirname(
      rootPath.replace(STRIP_MARKDOC_DIR_RE, "/root.md")
    );
    const node = {
      name,
      type,
      cname,
      filePath,
      rootPath,
      routePath,
      inline,
      owningDir
    };
    this._nodes.push(node);
    (_b = (_a = this._options).onAdd) == null ? void 0 : _b.call(_a, node);
    return node;
  }
  isNode(filePath) {
    filePath = this.normalizePath(filePath);
    return this.isAnyNode(filePath) && filePath.includes("@node");
  }
  isTag(filePath) {
    filePath = this.normalizePath(filePath);
    return this.isAnyNode(filePath) && !filePath.includes("@node");
  }
  isAnyNode(filePath) {
    filePath = this.normalizePath(filePath);
    return filePath.includes("@markdoc") && this._filter(filePath);
  }
  isOwnedBy(node, ownerFilePath) {
    const rootPath = this._getRootPath(ownerFilePath);
    const _node = isString(node) ? this.find(node) : node;
    return _node && rootPath.startsWith(_node.owningDir);
  }
  getOwnedNodes(ownerFilePath, type) {
    return Array.from(this._nodes).filter(
      (node) => (type === "*" || node.type === type) && this.isOwnedBy(node, ownerFilePath)
    );
  }
};
var validNodeNames = /* @__PURE__ */ new Set([
  "document",
  "heading",
  "paragraph",
  "blockquote",
  "hr",
  "image",
  "fence",
  "tag",
  "list",
  "item",
  "table",
  "thead",
  "tbody",
  "tr",
  "td",
  "th",
  "inline",
  "strong",
  "em",
  "s",
  "link",
  "code",
  "text",
  "hardbreak",
  "softbreak"
]);
function getValidNodeNames() {
  return validNodeNames;
}
function isValidMarkdownNodeName(name) {
  return validNodeNames.has(name);
}

// src/node/app/nodes/PageNodes.ts
import fs8 from "node:fs/promises";
import path9 from "node:path";
var MD_FILE_RE = /\.md($|\/)/;
var PAGE_LAYOUT_NAME_RE = /\+(.*?)\./;
var PageNodes = class extends FileNodes {
  init(app, options) {
    return super.init(app, {
      include: app.config.routes.pages.include,
      exclude: app.config.routes.pages.exclude,
      ...options
    });
  }
  async add(filePath) {
    var _a, _b;
    filePath = this.normalizePath(filePath);
    this.remove(filePath);
    const rootPath = this._getRootPath(filePath);
    const id = slash(rootPath);
    const ext = path9.posix.extname(rootPath);
    const fileContent = await fs8.readFile(filePath, "utf-8");
    const layoutName = await this.getLayoutName(filePath, fileContent);
    const hasLoader = this.hasLoader(fileContent);
    const route = this.resolveRoute(filePath);
    const page = {
      id,
      filePath,
      rootPath,
      ext,
      route,
      layouts: [],
      layoutName,
      hasLoader,
      context: {}
    };
    this._nodes.push(page);
    this._nodes = this._nodes.sort((a, b) => compareRoutes(a.route, b.route));
    (_b = (_a = this._options).onAdd) == null ? void 0 : _b.call(_a, page);
    return page;
  }
  async getLayoutName(pageFilePath, fileContent) {
    var _a;
    pageFilePath = this.normalizePath(pageFilePath);
    const frontmatter = MD_FILE_RE.test(pageFilePath) ? getFrontmatter(
      fileContent ?? await fs8.readFile(pageFilePath, "utf-8")
    ) : {};
    return ((_a = pageFilePath.match(PAGE_LAYOUT_NAME_RE)) == null ? void 0 : _a[1]) ?? frontmatter.layout ?? void 0;
  }
};

// src/node/app/nodes/AppNodes.ts
var AppNodes = class {
  constructor() {
    this.pages = new PageNodes();
    this.layouts = new LayoutNodes();
    this.endpoints = new EndpointNodes();
    this.markdoc = new MarkdocNodes();
  }
  async init(app) {
    await this.layouts.init(app, {
      onAdd: () => {
        this._resolvePageLayouts();
      },
      onRemove: (index) => {
        for (const page of this.pages) {
          if (page.layouts.includes(index)) {
            page.layouts = this.layouts.getOwnedLayoutIndicies(
              page.filePath,
              page.layoutName
            );
          }
        }
      }
    });
    await Promise.all([
      this.pages.init(app, {
        onAdd: (page) => {
          page.layouts = this.layouts.getOwnedLayoutIndicies(
            page.filePath,
            page.layoutName
          );
        }
      }),
      this.markdoc.init(app),
      this.endpoints.init(app)
    ]);
  }
  clear() {
    this.pages.clear();
    this.layouts.clear();
    this.endpoints.clear();
  }
  _resolvePageLayouts() {
    if (!this.pages.size)
      return;
    for (const page of this.pages) {
      page.layouts = this.layouts.getOwnedLayoutIndicies(
        page.filePath,
        page.layoutName
      );
    }
  }
};

// src/node/app/markdoc/render.ts
var voidElements = /* @__PURE__ */ new Set([
  "area",
  "base",
  "br",
  "col",
  "embed",
  "hr",
  "img",
  "input",
  "link",
  "meta",
  "param",
  "source",
  "track",
  "wbr"
]);
function renderMarkdocToHTML(node, config = {}) {
  if (typeof node === "string")
    return node;
  if (Array.isArray(node))
    return node.map((n) => renderMarkdocToHTML(n, config)).join("");
  if (node === null || typeof node !== "object")
    return "";
  const { name, attributes, children = [] } = node;
  if (attributes == null ? void 0 : attributes.__ignore)
    return "";
  if (!name)
    return renderMarkdocToHTML(children, config);
  let output = `<${name}`;
  const attr = config.attr ? (k, v) => config.attr(name, k, v) : (k, v) => `${k}="${String(v)}"`;
  for (const [k, v] of Object.entries(attributes ?? {})) {
    output += ` ${attr(k, v)}`;
  }
  output += ">";
  if (voidElements.has(name))
    return output;
  if (children.length)
    output += renderMarkdocToHTML(children, config);
  output += `</${name}>`;
  return output;
}

// src/node/app/markdoc/parse-markdown.ts
var cache = new LRUCache2({ max: 1024 });
var cacheK = new LRUCache2({ max: 1024 });
function clearMarkdownCache(file) {
  if (!file) {
    cache.reset();
  } else {
    cache.del(cacheK.get(file) ?? "");
  }
}
function parseMarkdown(app, filePath, source, opts = {}) {
  var _a, _b;
  const cacheKey = filePath + source;
  if (!opts.ignoreCache && cache.has(cacheKey))
    return cache.get(cacheKey);
  const ast = Markdoc2.parse(source, filePath);
  for (const transformer of opts.transformAst ?? []) {
    transformer({ filePath, ast, source });
  }
  const frontmatter = ast.attributes.frontmatter ? yaml.load(ast.attributes.frontmatter) : {};
  const nodeImports = app.markdoc.resolveOwnedImports(filePath);
  const config = app.markdoc.getOwnedConfig(filePath);
  const lastUpdated = Math.round(fs9.statSync(filePath).mtimeMs);
  const content = Markdoc2.transform(ast, {
    ...config,
    variables: {
      ...config.variables,
      frontmatter
    }
  });
  for (const transformer of opts.transformContent ?? []) {
    transformer({ filePath, frontmatter, content });
  }
  const stuff = {
    baseUrl: app.vite.resolved.base,
    filePath,
    appDir: app.dirs.app.path,
    highlight: opts.highlight,
    imports: /* @__PURE__ */ new Set(),
    links: /* @__PURE__ */ new Set(),
    headings: []
  };
  walkRenderTree(content, stuff, (node) => {
    forEachRenderNode(node, stuff);
    for (const transformer of opts.transformTreeNode ?? []) {
      transformer({ node, stuff });
    }
  });
  const { headings } = stuff;
  const title = ((_a = headings[0]) == null ? void 0 : _a.level) === 1 ? headings[0].title : null;
  const imports = Array.from(
    /* @__PURE__ */ new Set([...nodeImports, ...Array.from(stuff.imports)])
  );
  const meta = {
    title,
    headings,
    frontmatter,
    lastUpdated
  };
  const page = app.nodes.pages.find(filePath);
  if (page)
    mergeLayoutMeta(app, page, meta, opts);
  for (const transformer of opts.transformMeta ?? []) {
    transformer({ filePath, meta, imports, stuff });
  }
  let output = (((_b = opts.render) == null ? void 0 : _b.call(opts, { filePath, content, meta, imports, stuff })) ?? renderMarkdocToHTML(content)) || "";
  for (const transformer of opts.transformOutput ?? []) {
    output = transformer({ filePath, meta, code: output, imports, stuff });
  }
  const result = {
    meta,
    ast,
    filePath,
    output,
    stuff,
    content
  };
  cache.set(cacheKey, result);
  cacheK.set(filePath, cacheKey);
  return result;
}
function getFrontmatter(source) {
  const { data: frontmatter } = matter(source);
  return frontmatter;
}
function mergeLayoutMeta(app, page, meta, opts = {}) {
  var _a;
  const layoutFiles = page.layouts.map(
    (index) => app.nodes.layouts.getByIndex(index).filePath
  );
  for (const layoutFile of layoutFiles.reverse()) {
    if (!((_a = opts.filter) == null ? void 0 : _a.call(opts, layoutFile)))
      continue;
    const { ast: layoutAst, meta: layoutMeta } = parseMarkdown(
      app,
      layoutFile,
      fs9.readFileSync(layoutFile, { encoding: "utf-8" }),
      opts
    );
    let headingsPos = 0;
    for (const node of layoutAst.walk()) {
      if (node.type === "heading") {
        headingsPos += 1;
      } else if (node.attributes.content === "<slot />") {
        break;
      }
    }
    meta.title = meta.title ?? layoutMeta.title;
    meta.frontmatter = { ...layoutMeta.frontmatter, ...meta.frontmatter };
    meta.headings = [
      ...headingsPos > 0 ? layoutMeta.headings.slice(0, headingsPos) : layoutMeta.headings,
      ...meta.headings,
      ...headingsPos > 0 ? layoutMeta.headings.slice(headingsPos) : []
    ];
    if (layoutMeta.lastUpdated < meta.lastUpdated) {
      meta.lastUpdated = layoutMeta.lastUpdated;
    }
  }
}
function walkRenderTree(node, stuff, callback) {
  callback(node, stuff);
  if (node && typeof node !== "string" && node.children) {
    for (const child of node.children) {
      walkRenderTree(child, stuff, callback);
    }
  }
}
var codeNameRE = /^(code|Code)$/;
var fenceNameRE = /^(pre|Fence)$/;
var headingNameRE = /^(h\d|Heading)$/;
var linkNameRE = /^(a|link|Link)$/;
var importRE = /^import$/;
function forEachRenderNode(node, stuff) {
  if (node && typeof node !== "string") {
    const name = node.name;
    if (codeNameRE.test(name)) {
      transformCode(node);
    } else if (fenceNameRE.test(name)) {
      highlightCodeFences(node, stuff.highlight);
    } else if (headingNameRE.test(name)) {
      collectHeadings(node, stuff.headings);
    } else if (linkNameRE.test(name)) {
      resolveLinks(node, stuff);
    } else if (importRE.test(name)) {
      node.name = null;
      const file = node.attributes.file;
      if (file)
        stuff.imports.add(`import "${file}";`);
    }
  }
}
function transformCode(tag) {
  const isComponent = tag.name === "Code";
  const code = isComponent ? tag.attributes.content : tag.children[0];
  if (isComponent && typeof code === "string") {
    tag.attributes.code = escapeHTML(code);
    tag.children[0] = null;
    delete tag.attributes.content;
  }
}
var preTagRE = /<\/?pre(.*?)>/g;
var preTagStyleAttrRE = /<pre.*?style="(.*?)"/;
function highlightCodeFences(tag, highlight) {
  var _a;
  const isComponent = tag.name === "Fence";
  const lang = isComponent ? tag.attributes.language : tag.attributes["data-language"];
  const code = isComponent ? tag.attributes.content : tag.children[0];
  if (typeof code === "string") {
    const highlightedCode = highlight(code, lang);
    const styles = (_a = highlightedCode == null ? void 0 : highlightedCode.match(preTagStyleAttrRE)) == null ? void 0 : _a[1];
    const output = (highlightedCode == null ? void 0 : highlightedCode.replace(preTagRE, "")) ?? code;
    if (styles) {
      tag.attributes.style = (tag.attributes.style ?? "") + styles;
    }
    if (isComponent) {
      tag.attributes.lang = lang;
      tag.attributes.code = escapeHTML(code);
      if (highlightedCode)
        tag.attributes.highlightedCode = output;
      tag.children[0] = null;
      delete tag.attributes.language;
      delete tag.attributes["data-language"];
      delete tag.attributes.content;
    } else {
      tag.attributes.class = `${highlightedCode ? "highlight " : ""}lang-${lang}`;
      tag.children[0] = output;
    }
  }
}
function collectHeadings(tag, headings) {
  var _a;
  const child = tag.children[0];
  const title = typeof child === "string" ? child : (child == null ? void 0 : child.children[0]) ?? (child == null ? void 0 : child.attributes.content) ?? "";
  if (typeof title === "string") {
    const id = tag.attributes.id ?? slugify(title);
    const level = tag.attributes.level ?? Number(((_a = tag.name.match(/h(\d+)/)) == null ? void 0 : _a[1]) ?? 0);
    tag.attributes.id = id;
    tag.attributes.level = level;
    headings.push({
      title,
      id,
      level
    });
  }
}
function resolveLinks(tag, stuff) {
  const href = tag.attributes.href;
  if (!href)
    return;
  const internalLinkMatch = href.match(
    /^((?:.*)(?:\/|\.md|\.html|\.svelte|\.vue|\.jsx|\.tsx))(#.*)?$/
  );
  if (isLinkExternal(href, stuff.baseUrl)) {
    tag.attributes.target = "_blank";
    tag.attributes.rel = "noopener noreferrer";
    return;
  }
  if (internalLinkMatch) {
    const rawPath = decodeURI(internalLinkMatch == null ? void 0 : internalLinkMatch[1]);
    const rawHash = (internalLinkMatch == null ? void 0 : internalLinkMatch[2]) ?? "";
    const absolutePath = (rawPath == null ? void 0 : rawPath.startsWith("/")) ? path10.posix.resolve(stuff.appDir, "." + rawPath) : path10.posix.resolve(path10.posix.dirname(stuff.filePath), rawPath);
    const route = resolveStaticRouteFromFilePath(stuff.appDir, absolutePath);
    const resolvedHref = `${route}${rawHash}`;
    tag.attributes.href = resolvedHref;
    tag.attributes["data-prefetch"] = "";
    stuff.links.add(resolvedHref);
  }
}
var rControl = /[\u0000-\u001f]/g;
var rSpecial = /[\s~`!@#$%^&*()\-_+=[\]{}|\\;:"'“”‘’<>,.?/]+/g;
var rCombining = /[\u0300-\u036F]/g;
function slugify(str) {
  return str.normalize("NFKD").replace(rCombining, "").replace(rControl, "").replace(rSpecial, "-").replace(/-{2,}/g, "-").replace(/^-+|-+$/g, "").replace(/^(\d)/, "_$1").toLowerCase();
}

// src/node/app/plugins/vitebook-plugin.ts
import path12 from "node:path";

// src/node/app/build/build.ts
import fs10 from "node:fs";
import path11 from "node:path";

// src/node/app/create/app-factory.ts
import {
  mergeConfig
} from "vite";

// src/node/app/create/app-utils.ts
var getAppVersion = () => {
  return esmRequire()("vitebook/package.json").version;
};

// src/node/app/create/DisposalBin.ts
var DisposalBin = class {
  constructor() {
    this._disposal = [];
  }
  add(...disposeCallbacks) {
    for (const dispose of disposeCallbacks) {
      this._disposal.push(dispose);
    }
  }
  async empty() {
    for (const dispose of this._disposal) {
      await dispose();
    }
    this._disposal = [];
  }
};

// src/node/app/create/app-factory.ts
var createAppFactory = async (config, viteConfig, env) => {
  var _a, _b;
  await installURLPattern();
  const root = viteConfig.root ?? process.cwd();
  const resolvedConfig = resolveAppConfig(root, config);
  resolvedConfig.isBuild = env.command === "build";
  resolvedConfig.isSSR = !!((_a = viteConfig.build) == null ? void 0 : _a.ssr);
  const dirs = createAppDirectories(root, resolvedConfig);
  const version = getAppVersion();
  let plugins = viteConfig.plugins.flat().filter((plugin) => plugin && "vitebook" in plugin);
  plugins = [
    ...plugins.filter((plugin) => plugin.enforce === "pre"),
    ...plugins.filter((plugin) => !plugin.enforce),
    ...plugins.filter((plugin) => plugin.enforce === "post")
  ];
  const entry = ((_b = plugins.find((plugin) => {
    var _a2;
    return (_a2 = plugin.vitebook) == null ? void 0 : _a2.entry;
  })) == null ? void 0 : _b.vitebook.entry) ?? defaultEntry();
  Object.keys(entry).forEach((key) => entry[key] = normalizePath(entry[key]));
  const details = {
    version,
    dirs,
    entry,
    config: { ...resolvedConfig },
    vite: { env }
  };
  const app = {
    ...details,
    create: async () => {
      var _a2, _b2, _c, _d;
      const $app = {
        ...details,
        logger,
        vite: { user: viteConfig, env },
        context: /* @__PURE__ */ new Map(),
        nodes: new AppNodes(),
        markdoc: new MarkdocSchema(),
        disposal: new DisposalBin(),
        destroy: () => $app.disposal.empty()
      };
      for (const plugin of plugins) {
        const overrides = await ((_b2 = (_a2 = plugin.vitebook).config) == null ? void 0 : _b2.call(_a2, $app.config));
        if (overrides) {
          $app.config = mergeConfig(
            $app.config,
            overrides
          );
        }
      }
      for (const plugin of plugins) {
        await ((_d = (_c = plugin.vitebook) == null ? void 0 : _c.configureApp) == null ? void 0 : _d.call(_c, $app));
      }
      return $app;
    }
  };
  return app;
};
function defaultEntry() {
  const __require = esmRequire();
  return {
    client: __require.resolve(`vitebook/entry-client.js`),
    server: __require.resolve(`vitebook/entry-server.js`)
  };
}
function createAppEntries(app, { isSSR = false } = {}) {
  const entries = {};
  for (const page of app.nodes.pages) {
    const filename = resolvePageOutputFilename(app, page);
    entries[filename] = page.filePath;
  }
  for (const layout of app.nodes.layouts) {
    const filename = resolveLayoutOutputFilename(app, layout);
    entries[filename] = layout.filePath;
  }
  if (isSSR || app.config.isSSR) {
    for (const endpoint of app.nodes.endpoints) {
      const filename = resolveEndpointFilename(app, endpoint);
      entries[filename] = endpoint.filePath;
    }
  }
  return entries;
}
function resolvePageOutputFilename(app, page) {
  const name = app.dirs.app.relative(page.rootPath);
  return `pages/${name}`;
}
function resolveLayoutOutputFilename(app, layout) {
  const name = trimExt(
    app.dirs.app.relative(layout.rootPath).replace(/@layouts\//, "")
  );
  return `layouts/${name}`;
}
function resolveEndpointFilename(app, endpoint) {
  return trimExt(app.dirs.app.relative(endpoint.rootPath));
}

// src/node/app/http/polyfills.ts
var globals = {
  crypto: () => import("node:crypto"),
  URLPattern: async () => (await import("urlpattern-polyfill")).URLPattern,
  Headers: async () => (await import("undici")).Headers,
  ReadableStream: async () => (await import("node:stream/web")).ReadableStream,
  TransformStream: async () => (await import("node:stream/web")).TransformStream,
  WritableStream: async () => (await import("node:stream/web")).WritableStream,
  Request: async () => {
    const { Readable } = await import("node:stream");
    const { Request: Request2 } = await import("undici");
    const { Request: NodeFetchRequest } = await import("node-fetch");
    return class extends Request2 {
      formData() {
        return new NodeFetchRequest(this.url, {
          method: this.method,
          headers: this.headers,
          body: this.body && Readable.from(this.body)
        }).formData();
      }
    };
  },
  Response: async () => (await import("undici")).Response,
  fetch: async () => (await import("undici")).fetch
};
var installed = false;
async function installPolyfills() {
  if (installed)
    return;
  for (const name in globals) {
    if (!(name in globalThis)) {
      Object.defineProperty(globalThis, name, {
        enumerable: true,
        configurable: true,
        writable: true,
        value: await globals[name]()
      });
    }
  }
  installed = true;
}

// src/node/app/plugins/core/server-loader.ts
function createDataScriptTag(map) {
  const table = {};
  for (const id of map.keys()) {
    const data = map.get(id);
    if (data && Object.keys(data).length > 0) {
      table[id] = data;
    }
  }
  return [
    '<script id="__VBK_DATA__" type="application/json">',
    JSON.stringify(table),
    "<\/script>"
  ].join("");
}
function buildServerLoadedDataMap(map) {
  const data = /* @__PURE__ */ new Map();
  for (const id of map.keys()) {
    data.set(id, map.get(id).data ?? {});
  }
  return data;
}
function buildServerLoaderInput(url, page) {
  const match = execRouteMatch(url, page.route);
  return {
    pathname: url.pathname,
    page,
    route: page.route,
    params: match.groups,
    match
  };
}
async function loadPageServerOutput(url, app, page, moduleLoader) {
  const map = /* @__PURE__ */ new Map();
  const pathname = decodeURI(url.pathname);
  const input = buildServerLoaderInput(url, page);
  let redirect;
  await (async () => {
    const id = resolveDataAssetID(pathname);
    const output = await runModuleServerLoader(
      app,
      page.filePath,
      input,
      moduleLoader
    );
    map.set(id, output);
    if (output.redirect) {
      const path13 = isString(output.redirect) ? output.redirect : output.redirect.path;
      const statusCode = isString(output.redirect) ? 302 : output.redirect.statusCode ?? 302;
      const normalizedPath = !isLinkExternal(path13, app.vite.resolved.base) ? slash(path13) : path13;
      redirect = {
        path: normalizedPath,
        statusCode
      };
    }
  })();
  if (redirect)
    return { output: map, redirect };
  await Promise.all(
    page.layouts.map(async (index) => {
      const id = resolveDataAssetID(pathname, index);
      const layout = app.nodes.layouts.getByIndex(index);
      const output = await runModuleServerLoader(
        app,
        layout.filePath,
        input,
        moduleLoader
      );
      map.set(id, output);
    })
  );
  return { output: map, redirect };
}
var loadedCache = /* @__PURE__ */ new Map();
var cacheKeyBuilder = /* @__PURE__ */ new Map();
function clearServerLoaderCache(filePath) {
  cacheKeyBuilder.delete(filePath);
  loadedCache.delete(filePath);
}
async function runModuleServerLoader(app, filePath, input, moduleLoader) {
  if (!filePath)
    return {};
  const module = app.nodes.layouts.is(filePath) ? app.nodes.layouts.find(filePath) : app.nodes.pages.find(filePath);
  if (!module || !module.hasLoader) {
    clearServerLoaderCache(filePath);
    return {};
  }
  if (cacheKeyBuilder.has(filePath)) {
    const buildCacheKey2 = cacheKeyBuilder.get(filePath);
    const cacheKey = await buildCacheKey2(input);
    const cache2 = loadedCache.get(filePath);
    if (cacheKey && cache2 && cache2.has(cacheKey)) {
      return cache2.get(cacheKey);
    }
  }
  const { loader } = await moduleLoader(module.filePath);
  const output = await (loader == null ? void 0 : loader(input)) ?? {};
  const data = output.data;
  const buildCacheKey = output.cache;
  const isDataValid = !data || typeof data === "object";
  if (isDataValid && isFunction(buildCacheKey)) {
    const cacheKey = await buildCacheKey(input);
    if (cacheKey) {
      const cache2 = loadedCache.get(filePath) ?? /* @__PURE__ */ new Map();
      cache2.set(cacheKey, output);
      loadedCache.set(filePath, cache2);
    }
    cacheKeyBuilder.set(filePath, buildCacheKey);
  }
  if (!isDataValid) {
    app.logger.warn(
      "Received invalid data from loader (expected object).",
      [
        `
${kleur_default.bold("File Path:")} ${filePath}`,
        `${kleur_default.bold("Data Type:")} ${typeof output.data}`
      ].join("\n"),
      "\n"
    );
    output.data = {};
  }
  if (buildCacheKey && !isFunction(buildCacheKey)) {
    app.logger.warn(
      "Received invalid cache builder from loader (expected function).",
      [
        `
${kleur_default.bold("File Path:")} ${filePath}`,
        `${kleur_default.bold("Cache Type:")} ${typeof buildCacheKey}`
      ].join("\n"),
      "\n"
    );
  }
  return output;
}

// src/node/app/build/build.ts
async function build(app, clientBundle, serverBundle) {
  var _a, _b, _c, _d;
  if (app.nodes.pages.size === 0) {
    console.log(kleur_default.bold(`\u2753 No pages were resolved`));
    return;
  }
  await installPolyfills();
  const ssrProtocol = app.vite.resolved.server.https ? "https" : "http";
  const ssrOrigin = `${ssrProtocol}://ssr`;
  const pages = app.nodes.pages.toArray();
  const entries = createAppEntries(app, { isSSR: true });
  const entryFilenames = Object.keys(entries);
  const viteManifestPath = app.dirs.client.resolve("vite-manifest.json");
  const { chunks, assets } = collectOutput(clientBundle);
  const entryChunk = chunks.find(
    (chunk) => chunk.isEntry && /^_immutable\/entry-/.test(chunk.fileName)
  );
  const appChunk = chunks.find(
    (chunk) => chunk.isEntry && /^_immutable\/app-/.test(chunk.fileName)
  );
  const appCSSAsset = assets.find((asset) => asset.fileName.endsWith(".css"));
  const bundles = {
    entries,
    client: {
      output: clientBundle,
      entryChunk,
      appChunk,
      appCSSAsset,
      chunks,
      assets,
      viteManifest: JSON.parse(
        await fs10.promises.readFile(viteManifestPath, "utf-8")
      )
    },
    server: {
      output: serverBundle,
      chunks: collectChunks(serverBundle)
    }
  };
  const build3 = {
    entries,
    links: /* @__PURE__ */ new Map(),
    badLinks: /* @__PURE__ */ new Map(),
    data: /* @__PURE__ */ new Map(),
    redirects: /* @__PURE__ */ new Map(),
    endpoints: /* @__PURE__ */ new Map(),
    loaded: /* @__PURE__ */ new Map(),
    renders: /* @__PURE__ */ new Map()
  };
  const $2 = getBuildAdapterUtils(app, bundles, build3);
  const adapterFactory = isFunction(app.config.build.adapter) ? app.config.build.adapter : createAutoBuildAdapter(app.config.build.adapter);
  const adapter = await adapterFactory(app, bundles, build3, $2);
  for (const endpoint of app.nodes.endpoints) {
    build3.endpoints.set(
      `/${path11.dirname(app.dirs.app.relative(endpoint.rootPath))}/`,
      endpoint
    );
  }
  const specialCharsRE = /\$|#|\[|\]|{|}|:/g;
  const resolveServerChunkPath = (filePath) => {
    var _a2;
    return app.dirs.server.resolve(
      `${(_a2 = entryFilenames.find((name) => entries[name] === filePath)) == null ? void 0 : _a2.replace(specialCharsRE, "_")}.js`
    );
  };
  globalThis.fetch = (input, init2) => {
    if (typeof input === "string" && input.startsWith("/api")) {
      const url = new URL(`${ssrOrigin}${input}`);
      const match = matchRouteInfo(url, app.nodes.endpoints.toArray());
      if (!match) {
        return Promise.resolve(new Response("Not found", { status: 404 }));
      }
      const endpoint = app.nodes.endpoints.getByIndex(match.index);
      return handleRequest(
        new Request(url, init2),
        match.route.pattern,
        () => {
          throw new Error("Can not resolve `clientAddres` during SSR");
        },
        () => import(resolveServerChunkPath(endpoint.filePath))
      );
    }
    return fetch(input, init2);
  };
  async function loadServerOutput(url, page) {
    var _a2, _b2, _c2;
    const pathname = $2.normalizeURL(url).pathname;
    if (build3.loaded.has(pathname)) {
      return build3.loaded.get(pathname);
    }
    await ((_a2 = adapter.startLoadingPage) == null ? void 0 : _a2.call(adapter, pathname, page));
    const result = await loadPageServerOutput(
      url,
      app,
      page,
      (filePath) => import(resolveServerChunkPath(filePath))
    );
    build3.loaded.set(pathname, result);
    if (result.redirect) {
      const pathname2 = $2.normalizeURL(url).pathname;
      build3.redirects.set(pathname2, {
        from: pathname2,
        to: result.redirect.path,
        filename: $2.resolveHTMLFilename(url),
        html: $2.createRedirectMetaTag(result.redirect.path),
        statusCode: result.redirect.statusCode
      });
      await ((_b2 = adapter == null ? void 0 : adapter.finishLoadingPage) == null ? void 0 : _b2.call(
        adapter,
        pathname2,
        page,
        result.output,
        result.redirect.path
      ));
      return result;
    }
    for (const id of result.output.keys()) {
      const content = result.output.get(id).data ?? {};
      if (Object.keys(content).length > 0) {
        const serializedContent = JSON.stringify(content);
        const contentHash = $2.hash(serializedContent);
        build3.data.set(id, {
          data: content,
          idHash: $2.hash(id),
          contentHash,
          filename: $2.resolveDataFilename(contentHash),
          serializedData: JSON.stringify(content)
        });
      }
    }
    await ((_c2 = adapter == null ? void 0 : adapter.finishLoadingPage) == null ? void 0 : _c2.call(
      adapter,
      pathname,
      page,
      result.output,
      result.redirect
    ));
    return result;
  }
  const serverEntryPath = app.dirs.server.resolve("entry.js");
  const { render } = await import(serverEntryPath);
  const validPathname = /(\/|\.html)$/;
  async function buildPage(url, page) {
    var _a2, _b2, _c2;
    const pathname = $2.normalizeURL(url).pathname;
    if (build3.links.has(pathname) || build3.badLinks.has(pathname)) {
      return;
    }
    if (!validPathname.test(pathname)) {
      build3.badLinks.set(pathname, {
        page,
        reason: "malformed URL pathname"
      });
      return;
    }
    build3.links.set(pathname, page);
    await ((_a2 = adapter.startRenderingPage) == null ? void 0 : _a2.call(adapter, pathname, page));
    const loadedOutput = await loadServerOutput(url, page);
    if (loadedOutput.redirect) {
      const location = loadedOutput.redirect.path;
      await buildPageFromHref(page, location);
      await ((_b2 = adapter.finishRenderingPage) == null ? void 0 : _b2.call(adapter, pathname, page, {
        redirect: loadedOutput.redirect
      }));
      return;
    }
    const serverData = buildServerLoadedDataMap(loadedOutput.output);
    const ssr = await render(url, { data: serverData });
    const result = {
      filename: $2.resolveHTMLFilename(url),
      page,
      ssr,
      dataAssetIds: new Set(serverData.keys())
    };
    build3.renders.set(pathname, result);
    await ((_c2 = adapter.finishRenderingPage) == null ? void 0 : _c2.call(adapter, pathname, page, result));
    const hrefs = $2.crawl(ssr.html);
    for (let i = 0; i < hrefs.length; i++) {
      await buildPageFromHref(page, hrefs[i]);
    }
  }
  const notFoundRE = /\/@404\./;
  async function buildPageFromHref(page, href) {
    if (href.startsWith("#") || $2.isLinkExternal(href))
      return;
    const url = new URL(`${ssrOrigin}${$2.slash(href)}`);
    const pathname = $2.normalizeURL(url).pathname;
    if (build3.links.has(pathname) || build3.badLinks.has(pathname))
      return;
    const { index } = matchRouteInfo(url, pages) ?? {};
    const foundPage = !isUndefined(index) && index >= 0 ? pages[index] : null;
    if (foundPage && !notFoundRE.test(foundPage.id)) {
      await buildPage(url, foundPage);
      return;
    }
    build3.badLinks.set(pathname, {
      page,
      reason: "no matching route (404)"
    });
  }
  for (const page of pages.filter((page2) => !page2.route.dynamic).reverse()) {
    await buildPage(
      new URL(`${ssrOrigin}${cleanRoutePath(page.route.pathname)}`),
      page
    );
  }
  await ((_a = adapter.startRenderingPages) == null ? void 0 : _a.call(adapter));
  for (const entry of app.config.routes.entries) {
    const url = new URL(`${ssrOrigin}${$2.slash(entry)}`);
    const { index } = matchRouteInfo(url, pages) ?? {};
    if (index) {
      const page = pages[index];
      await buildPage(url, page);
    } else {
      build3.badLinks.set(entry, {
        reason: "no matching route (404)"
      });
    }
  }
  await ((_b = adapter.finishRenderingPages) == null ? void 0 : _b.call(adapter));
  await ((_c = adapter.write) == null ? void 0 : _c.call(adapter));
  await ((_d = adapter.close) == null ? void 0 : _d.call(adapter));
}
function collectChunks(bundle) {
  const chunks = [];
  for (const value of Object.values(bundle)) {
    if (value.type === "chunk") {
      chunks.push(value);
    }
  }
  return chunks;
}
function collectOutput(bundle) {
  const chunks = [];
  const assets = [];
  for (const value of Object.values(bundle)) {
    if (value.type === "asset") {
      assets.push(value);
    } else {
      chunks.push(value);
    }
  }
  return { chunks, assets };
}

// src/node/app/build/bundle.ts
import { build as build2 } from "vite";
async function createServerBundle(callback) {
  await build2({
    build: { ssr: true },
    plugins: [
      {
        name: "vitebook-server-bundle",
        writeBundle(_, bundle) {
          callback(bundle);
        }
      }
    ]
  });
}
function resolveBuildConfig(app) {
  const ssr = app.config.isSSR;
  const immutableDir = "_immutable";
  const input = {
    entry: ssr ? app.entry.server : app.entry.client,
    app: app.config.client.app,
    ...createAppEntries(app)
  };
  return {
    appType: "spa",
    logLevel: "warn",
    publicDir: ssr ? false : app.dirs.public.path,
    esbuild: { treeShaking: !ssr },
    build: {
      emptyOutDir: true,
      ssr,
      target: ssr ? "node16" : void 0,
      manifest: !ssr && `vite-manifest.json`,
      ssrManifest: false,
      cssCodeSplit: false,
      assetsDir: `${immutableDir}/assets`,
      minify: !ssr && !app.config.isDebug,
      polyfillModulePreload: false,
      outDir: ssr ? app.dirs.root.relative(app.dirs.server.path) : app.dirs.root.relative(app.dirs.client.path),
      rollupOptions: {
        input,
        output: {
          format: "esm",
          entryFileNames: ssr ? `[name].js` : `${immutableDir}/[name]-[hash].js`,
          chunkFileNames: ssr ? "chunks/[name].js" : `${immutableDir}/chunks/[name]-[hash].js`,
          assetFileNames: ssr ? "" : `${immutableDir}/assets/[name]-[hash][extname]`,
          manualChunks: extendManualChunks()
        },
        preserveEntrySignatures: "strict"
      }
    }
  };
}

// src/node/app/plugins/core/handle-data.ts
async function handleDataRequest(url, app, res) {
  const { url: route, layoutIndex } = parseDataAssetURL(url);
  const match = matchRouteInfo(route, app.nodes.pages.toArray());
  if (!match) {
    res.statusCode = 404;
    res.end("Not found");
    return;
  }
  const page = app.nodes.pages.getByIndex(match.index);
  const module = layoutIndex >= 0 ? app.nodes.layouts.getByIndex(layoutIndex) : page;
  const output = await runModuleServerLoader(
    app,
    module.filePath,
    buildServerLoaderInput(route, page),
    app.vite.server.ssrLoadModule
  );
  if (output.redirect) {
    output.data = { ...output.data, __redirect__: output.redirect };
  }
  res.statusCode = 200;
  res.setHeader("Content-Type", "application/json");
  res.end(JSON.stringify(output.data ?? {}));
}

// src/node/app/plugins/core/handle-endpoint.ts
async function handleEndpoint(base, url, app, req, res, loader) {
  const match = matchRouteInfo(url, app.nodes.endpoints.toArray());
  if (!match) {
    res.statusCode = 404;
    res.end("Not found");
    return;
  }
  const endpoint = app.nodes.endpoints.getByIndex(match.index);
  const handler = createHTTPRequestHandler(
    () => endpoint.route.pattern,
    () => loader(endpoint),
    {
      getBase: () => base,
      handleUnknownError(_req, _res, error) {
        throw error;
      }
    }
  );
  return handler(req, res);
}

// src/node/app/plugins/core/handle-page.ts
async function handlePageRequest(url, app, req, res) {
  const pathname = decodeURI(url.pathname);
  const index = readIndexHtmlFile(app);
  const transformedIndex = await app.vite.server.transformIndexHtml(
    pathname,
    index,
    req.originalUrl
  );
  const { render } = await app.vite.server.ssrLoadModule(
    app.entry.server
  );
  const match = matchRouteInfo(url, app.nodes.pages.toArray());
  const page = app.nodes.pages.getByIndex((match == null ? void 0 : match.index) ?? -1);
  if (!page) {
    res.statusCode = 404;
    res.end("Not found");
    return;
  }
  const { output: serverOutput, redirect } = await loadPageServerOutput(
    url,
    app,
    page,
    app.vite.server.ssrLoadModule
  );
  if (redirect) {
    res.statusCode = redirect.statusCode;
    res.setHeader("Location", redirect.path).end();
    return;
  }
  const serverData = buildServerLoadedDataMap(serverOutput);
  const dataScript = createDataScriptTag(serverData);
  const { html: appHtml, head } = await render(url, { data: serverData });
  const appFilePath = app.vite.server.moduleGraph.getModuleById(`/${virtualModuleId.app}`).importedModules.values().next().value.file;
  const stylesMap = await Promise.all(
    [
      appFilePath,
      ...page.layouts.map(
        (index2) => app.nodes.layouts.getByIndex(index2).filePath
      ),
      page.filePath
    ].map((file) => getStylesByFile(app.vite.server, file))
  );
  const styles = [
    '<style id="__VBK_SSR_STYLES__" type="text/css">',
    stylesMap.map(Object.values).flat().join("\n"),
    "</style>"
  ].join("\n");
  const html = transformedIndex.replace(`<!--@vitebook/head-->`, head + styles).replace(`<!--@vitebook/app-->`, appHtml).replace("<!--@vitebook/body-->", dataScript);
  res.statusCode = 200;
  res.setHeader("Content-Type", "text/html");
  res.end(html);
}
var styleRE = /\.(css|less|sass|scss|styl|stylus|pcss|postcss)$/;
async function getStylesByFile(server, file) {
  const files = await server.moduleGraph.getModulesByFile(file);
  const node = Array.from(files ?? [])[0];
  if (!node)
    throw new Error(`Could not find node for ${file}`);
  const deps = /* @__PURE__ */ new Set();
  await findModuleDeps(server, node, deps);
  const styles = {};
  for (const dep of deps) {
    const parsed = new URL(dep.url, "http://localhost/");
    const query = parsed.searchParams;
    if (styleRE.test(dep.file) || query.has("svelte") && query.get("type") === "style") {
      try {
        const mod = await server.ssrLoadModule(dep.url);
        styles[dep.url] = mod.default;
      } catch {
      }
    }
  }
  return styles;
}
async function findModuleDeps(server, node, deps) {
  const edges = [];
  async function add(node2) {
    if (!deps.has(node2)) {
      deps.add(node2);
      await findModuleDeps(server, node2, deps);
    }
  }
  async function addByUrl(url) {
    const node2 = await server.moduleGraph.getModuleByUrl(url);
    if (node2)
      await add(node2);
  }
  if (node.ssrTransformResult) {
    if (node.ssrTransformResult.deps) {
      node.ssrTransformResult.deps.forEach((url) => edges.push(addByUrl(url)));
    }
  } else {
    node.importedModules.forEach((node2) => edges.push(add(node2)));
  }
  await Promise.all(edges);
}

// src/node/app/plugins/core/dev-server.ts
function configureDevServer(app, server) {
  removeHtmlMiddlewares(server.middlewares);
  const fetch2 = globalThis.fetch;
  const protocol = server.config.server.https ? "https" : "http";
  const loader = (endpoint) => app.vite.server.ssrLoadModule(endpoint.filePath);
  let origin;
  globalThis.fetch = (input, init2) => {
    var _a;
    return fetch2(
      typeof input === "string" && input.startsWith("/api") ? `${origin ??= noendslash(
        ((_a = server.resolvedUrls) == null ? void 0 : _a.local[0]) ?? `${protocol}://localhost:5173`
      )}${input}` : input,
      init2
    );
  };
  server.middlewares.use(async (req, res, next) => {
    try {
      if (!req.url || !req.method) {
        throw new Error("Incomplete request");
      }
      const base = `${protocol}://${req.headers[":authority"] || req.headers.host}`;
      const url = new URL(base + req.url);
      const decodedUrl = decodeURI(new URL(base + req.url).pathname);
      if (decodedUrl.startsWith("/api")) {
        return await handleEndpoint(base, url, app, req, res, loader);
      }
      if (decodedUrl.startsWith(DATA_ASSET_BASE_PATH)) {
        return await handleDataRequest(url, app, res);
      }
      if (decodedUrl.endsWith("/") || decodedUrl.endsWith(".html")) {
        url.pathname = url.pathname.replace("/index.html", "/");
        return await handlePageRequest(url, app, req, res);
      }
    } catch (error) {
      handleDevServerError(app, req, res, error);
      return;
    }
    next();
  });
}
function removeHtmlMiddlewares(server) {
  const middlewares = ["viteIndexHtmlMiddleware", "viteSpaFallbackMiddleware"];
  for (let i = server.stack.length - 1; i > 0; i--) {
    if (middlewares.includes(server.stack[i].handle.name)) {
      server.stack.splice(i, 1);
    }
  }
}
function handleDevServerError(app, req, res, e) {
  const error = coalesceToError(e);
  app.logger.error(
    error.message,
    [
      `
${kleur_default.bold("URL:")} ${req.url ?? "?"}`,
      `${kleur_default.bold("METHOD:")} ${req.method ?? "?"}`,
      "",
      ""
    ].join("\n"),
    error.stack,
    "\n"
  );
  res.statusCode = 500;
  res.end(error.stack);
}

// src/node/app/plugins/core/preview-server.ts
async function configurePreviewServer(app, server) {
  await installPolyfills();
  const protocol = app.vite.resolved.server.https || app.vite.resolved.preview.https ? "https" : "http";
  const loader = (endpoint) => {
    return import(app.dirs.server.resolve(app.dirs.app.relative(endpoint.rootPath)).replace(/\.ts$/, ".js"));
  };
  server.middlewares.use(async (req, res, next) => {
    try {
      if (!req.url || !req.method) {
        throw new Error("Incomplete request");
      }
      const base = `${protocol}://${req.headers[":authority"] || req.headers.host}`;
      const url = new URL(base + req.url);
      const decodedUrl = decodeURI(new URL(base + req.url).pathname);
      if (decodedUrl.startsWith("/api")) {
        return await handleEndpoint(base, url, app, req, res, loader);
      }
    } catch (error) {
      handleDevServerError(app, req, res, error);
      return;
    }
    next();
  });
}

// src/node/app/plugins/markdown/markdown-plugin.ts
import { createFilter as createFilter3 } from "@rollup/pluginutils";
import { readFile } from "fs/promises";
import { toHtml } from "hast-util-to-html";

// src/node/app/plugins/nodes/nodes-hmr.ts
import fs11 from "fs/promises";
function handleNodesHMR(app) {
  const pages = app.nodes.pages;
  const layouts = app.nodes.layouts;
  const endpoints = app.nodes.endpoints;
  const server = app.vite.server;
  const isPage = (filePath) => pages.is(filePath);
  const isLayout = (filePath) => layouts.is(filePath);
  const isEndpoint = (filePath) => endpoints.is(filePath);
  function clearLayoutChildrenMarkdownCache(layoutFilePath) {
    const layoutIndex = layouts.findIndex(layoutFilePath);
    for (const page of pages) {
      if (page.layouts.includes(layoutIndex)) {
        clearMarkdownCache(page.filePath);
        invalidatePageModule(server, page);
      }
    }
  }
  onFileEvent(isEndpoint, "add", async (filePath) => {
    endpoints.add(filePath);
  });
  onFileEvent(isEndpoint, "unlink", async (filePath) => {
    endpoints.remove(filePath);
  });
  onFileEvent(isPage, "add", async (filePath) => {
    pages.add(filePath);
    return { reload: true };
  });
  onFileEvent(isPage, "unlink", async (filePath) => {
    pages.remove(filePath);
    return { reload: true };
  });
  onFileEvent(isPage, "change", async (filePath) => {
    const page = pages.find(filePath);
    const layoutName = await pages.getLayoutName(filePath);
    const hasLoader = pages.hasLoader(await fs11.readFile(filePath, "utf-8"));
    if (page && page.layoutName !== layoutName) {
      page.layouts = layouts.getOwnedLayoutIndicies(
        page.filePath,
        page.layoutName
      );
      return { reload: true };
    }
    if (page && page.hasLoader !== hasLoader) {
      page.hasLoader = hasLoader;
      return { reload: true };
    }
    if (hasLoader) {
      clearServerLoaderCache(filePath);
    }
    return null;
  });
  onFileEvent(isLayout, "add", async (filePath) => {
    layouts.add(filePath);
    clearLayoutChildrenMarkdownCache(filePath);
    return { reload: true };
  });
  onFileEvent(isLayout, "change", async (filePath) => {
    const layout = layouts.find(filePath);
    const hasLoader = layouts.hasLoader(await fs11.readFile(filePath, "utf-8"));
    if (layout && layout.hasLoader !== hasLoader) {
      layout.hasLoader = hasLoader;
      return { reload: true };
    }
    if (hasLoader) {
      clearServerLoaderCache(filePath);
    }
    return null;
  });
  onFileEvent(isLayout, "unlink", async (filePath) => {
    clearLayoutChildrenMarkdownCache(filePath);
    layouts.remove(filePath);
    return { reload: true };
  });
  function onFileEvent(test, eventName, handler) {
    server.watcher.on(eventName, async (path13) => {
      const filePath = normalizePath(path13);
      if (!test(filePath))
        return;
      const { reload } = await handler(filePath) ?? {};
      if (reload) {
        fullReload();
      }
    });
  }
  function fullReload() {
    invalidateModuleByID(virtualModuleRequestPath.pages);
    invalidateModuleByID(virtualModuleRequestPath.layouts);
    server.ws.send({ type: "full-reload" });
  }
  function invalidateModuleByID(id) {
    const mod = server.moduleGraph.getModuleById(id);
    if (mod)
      server.moduleGraph.invalidateModule(mod);
  }
}
function invalidatePageModule(server, page) {
  var _a;
  const module = (_a = server.moduleGraph.getModulesByFile(page.filePath)) == null ? void 0 : _a.values().next();
  if (module == null ? void 0 : module.value)
    server.moduleGraph.invalidateModule(module.value);
}

// src/node/app/plugins/markdown/hmr.ts
function handleMarkdownHMR(app) {
  const schema = app.markdoc;
  const nodes = app.nodes.markdoc;
  const isNode = (filePath) => nodes.isAnyNode(filePath);
  onFileEvent(isNode, "add", async (filePath) => {
    nodes.add(filePath);
    for (const page of app.nodes.pages) {
      if (nodes.isOwnedBy(filePath, page.filePath)) {
        clearMarkdownCache(page.filePath);
        invalidateFile(page.filePath);
      }
    }
    return { reload: true };
  });
  onFileEvent(isNode, "unlink", async (filePath) => {
    nodes.remove(filePath);
    const files = schema.hmrFiles.get(filePath);
    if (files) {
      for (const file of files) {
        clearMarkdownCache(file);
        invalidateFile(file);
      }
    }
    schema.hmrFiles.delete(filePath);
    return { reload: true };
  });
  function onFileEvent(test, eventName, handler) {
    app.vite.server.watcher.on(eventName, async (path13) => {
      const filePath = normalizePath(path13);
      if (!test(filePath))
        return;
      const { reload } = await handler(filePath) ?? {};
      if (reload) {
        app.vite.server.ws.send({ type: "full-reload" });
      }
    });
  }
  function invalidateFile(filePath) {
    var _a;
    (_a = app.vite.server.moduleGraph.getModulesByFile(filePath)) == null ? void 0 : _a.forEach((mod) => app.vite.server.moduleGraph.invalidateModule(mod));
  }
}

// src/node/app/plugins/markdown/markdown-plugin.ts
function markdownPlugin() {
  let app;
  let filter;
  let currentPage = void 0;
  let parse;
  let highlight = null;
  return {
    name: "@vitebook/markdown",
    enforce: "pre",
    vitebook: {
      async configureApp(_app) {
        var _a;
        app = _app;
        const config = app.config.markdown;
        const { include, exclude, hastToHtml, highlighter, ...parseOptions } = config;
        highlight = typeof highlighter === "function" ? highlighter : null;
        parse = (filePath, content) => parseMarkdown(app, filePath, content, {
          ignoreCache: false,
          filter,
          highlight: (code, lang) => highlight == null ? void 0 : highlight(code, lang),
          ...parseOptions
        });
        filter = createFilter3(include, exclude);
        if (highlighter === "starry-night") {
          try {
            const mod = await import("@wooorm/starry-night");
            const starryNight = await mod.createStarryNight(mod.all);
            highlight = (code, lang) => {
              const scope = starryNight.flagToScope(lang);
              if (!scope)
                return "";
              const tree = starryNight.highlight(code, scope);
              return toHtml(tree, hastToHtml);
            };
          } catch (error) {
            app.logger.error(
              `Failed to import \`@wooorm/starry-night\`, is it installed?`,
              `

${kleur_default.bold("npm install @wooorm/starry-night")}`,
              `

${error}`
            );
          }
        }
        if (highlighter === "shiki") {
          try {
            const mod = await import("shiki");
            const shiki = await mod.getHighlighter(config.shiki);
            const theme = (typeof config.shiki.theme === "string" ? config.shiki.theme : (_a = config.shiki.theme) == null ? void 0 : _a.name) ?? "material-palenight";
            highlight = (code, lang) => {
              const tokens = shiki.codeToThemedTokens(code, lang);
              return mod.renderToHtml(tokens, {
                fg: shiki.getForegroundColor(theme),
                bg: shiki.getBackgroundColor(theme)
              });
            };
          } catch (error) {
            app.logger.error(
              `Failed to import \`shiki\`, is it installed?`,
              `

${kleur_default.bold("npm install shiki")}`,
              `

${error}`
            );
          }
        }
        await app.markdoc.init(app);
      }
    },
    async configureServer(server) {
      handleMarkdownHMR(app);
      server.ws.on("vitebook::page_change", ({ rootPath }) => {
        const filePath = app.dirs.root.resolve(rootPath);
        currentPage = app.nodes.pages.find(filePath);
      });
    },
    transform(content, id) {
      if (filter(id)) {
        const { output } = parse(id, content);
        return output;
      }
      return null;
    },
    async handleHotUpdate(ctx) {
      const { file, server, read } = ctx;
      if (filter(file)) {
        const content = await read();
        const layoutIndex = app.nodes.layouts.findIndex(file);
        const isLayoutFile = layoutIndex >= 0;
        if (isLayoutFile && (currentPage == null ? void 0 : currentPage.layouts.includes(layoutIndex))) {
          clearMarkdownCache(currentPage.filePath);
          invalidatePageModule(server, currentPage);
          const { meta: meta2 } = parse(
            currentPage.filePath,
            await readFile(currentPage.filePath, { encoding: "utf-8" })
          );
          handleMarkdownMetaHMR(server, currentPage.filePath, meta2);
        }
        const { output, meta } = parse(file, content);
        ctx.read = () => output;
        if (!isLayoutFile)
          handleMarkdownMetaHMR(server, file, meta);
      }
    }
  };
}
function handleMarkdownMetaHMR(server, filePath, meta) {
  server.ws.send({
    type: "custom",
    event: "vitebook::md_meta",
    data: { filePath, meta }
  });
}

// src/node/app/plugins/nodes/load.ts
var STRIP_URL_PATTERN_QUOTES_RE = /"new URLPattern(.*?)"/g;
function loadPagesModule(pages) {
  return `export default ${stripImportQuotesFromJson(
    prettyJsonStr(
      pages.map((page) => ({
        rootPath: page.rootPath,
        route: {
          ...page.route,
          pattern: void 0
        },
        ext: page.ext,
        layoutName: page.layoutName,
        layouts: page.layouts,
        context: Object.keys(page.context).length > 0 ? page.context : void 0,
        loader: `() => import('${page.id}')`
      }))
    )
  )}`.replace(STRIP_URL_PATTERN_QUOTES_RE, "new URLPattern$1");
}
function loadLayoutsModule(layouts) {
  return `export default ${stripImportQuotesFromJson(
    prettyJsonStr(
      layouts.map((layout) => {
        return {
          name: layout.name,
          rootPath: layout.rootPath,
          loader: `() => import('${layout.id}')`
        };
      })
    )
  )}`;
}

// src/node/app/plugins/nodes/nodes-plugin.ts
function nodesPlugin() {
  let app;
  return {
    name: "@vitebook/nodes",
    enforce: "pre",
    vitebook: {
      async configureApp(_app) {
        app = _app;
        await app.nodes.init(app);
      }
    },
    async configureServer(server) {
      server.watcher.add(app.dirs.app.path);
      handleNodesHMR(app);
    },
    async load(id) {
      if (id === virtualModuleRequestPath.pages) {
        return loadPagesModule(app.nodes.pages.toArray());
      }
      if (id === virtualModuleRequestPath.layouts) {
        return loadLayoutsModule(app.nodes.layouts.toArray());
      }
      return null;
    }
  };
}

// src/node/app/plugins/vitebook-plugin.ts
var clientPackages = [
  "vitebook",
  "@vitebook/svelte",
  "urlpattern-polyfill/urlpattern"
];
function vitebookPlugin(config = {}) {
  let app, appFactory, viteConfig;
  let isFirstBuild = true, clientBundle = null, serverBundle = null;
  return [
    {
      name: "vitebook",
      enforce: "pre",
      async config(viteConfig2, env) {
        appFactory = await createAppFactory(config, viteConfig2, env);
        app = await appFactory.create();
        return {
          ...resolveBuildConfig(app),
          envPrefix: "PUBLIC_",
          resolve: { alias: virtualAliases },
          optimizeDeps: { exclude: clientPackages },
          ssr: { noExternal: clientPackages },
          server: {
            fs: {
              allow: [
                path12.posix.dirname(app.entry.client),
                path12.posix.dirname(app.entry.server),
                app.dirs.cwd.path,
                app.dirs.root.path,
                app.dirs.app.path,
                app.dirs.public.path,
                app.dirs.build.path,
                app.dirs.tmp.path,
                app.dirs.cwd.resolve("node_modules"),
                app.dirs.workspace.resolve("node_modules")
              ]
            }
          }
        };
      },
      async configResolved(_viteConfig) {
        viteConfig = _viteConfig;
        app.vite.resolved = _viteConfig;
      },
      async configureServer(server) {
        await installPolyfills();
        app.vite.server = server;
        return () => {
          configureDevServer(app, server);
        };
      },
      async configurePreviewServer(server) {
        await configurePreviewServer(app, server);
      },
      resolveId(id) {
        if (id === virtualModuleRequestPath.client) {
          return { id: app.entry.client };
        }
        if (id === virtualModuleRequestPath.app || id === virtualModuleRequestPath.noop) {
          return id;
        }
        return null;
      },
      async load(id) {
        if (id === virtualModuleRequestPath.app) {
          const id2 = app.config.client.app;
          const baseUrl = app.vite.resolved.base;
          const configs = app.config.client.configFiles;
          return [
            `import * as App from "${id2}";`,
            "",
            configs.map(
              (id3, i) => `import { configureApp as configureApp$${i} } from "${id3}";`
            ).join("\n"),
            "",
            `export default {
            id: "${id2}",
            baseUrl: "${baseUrl}",
            module: App,
            configs: [${configs.map((_, i) => `configureApp$${i}`).join(", ")}]
          };`
          ].join("\n");
        }
        if (id === virtualModuleRequestPath.noop) {
          return `export default function() {};`;
        }
        return null;
      },
      async buildStart() {
        if (app.config.isSSR)
          return;
        clientBundle = null;
        if (app.config.isBuild) {
          rimraf(appFactory.dirs.tmp.path);
          rimraf(appFactory.dirs.build.path);
          if (!isFirstBuild) {
            app = await appFactory.create();
            app.vite.resolved = viteConfig;
          }
          isFirstBuild = false;
        }
      },
      async writeBundle(_, bundle) {
        if (app.config.isSSR)
          return;
        clientBundle = bundle;
        await createServerBundle((bundle2) => {
          serverBundle = bundle2;
        });
      },
      async closeBundle() {
        if (app.config.isSSR || !clientBundle || !app || !serverBundle) {
          app == null ? void 0 : app.destroy();
          return;
        }
        await build(app, clientBundle, serverBundle);
        clientBundle = null;
        serverBundle = null;
        app.destroy();
      },
      generateBundle(_, bundle) {
        if (app.config.isSSR) {
          for (const name in bundle) {
            if (bundle[name].type === "asset") {
              delete bundle[name];
            }
          }
        }
      }
    },
    markdownPlugin(),
    nodesPlugin()
  ];
}

// src/node/index.ts
import { default as default2 } from "@markdoc/markdoc";
export {
  AppNodes,
  DATA_ASSET_BASE_PATH,
  EXTERNAL_URL_RE,
  EndpointNodes,
  FileNodes,
  HASH_RE,
  LayoutNodes,
  LoggerColor,
  LoggerIcon,
  default2 as Markdoc,
  MarkdocNodes,
  PageNodes,
  QUERY_RE,
  VM_PREFIX,
  calcRoutePathScore,
  checksumFile,
  cleanRoutePath,
  cleanUrl,
  coalesceToError,
  comparePaths,
  compareRoutes,
  copyDir,
  copyFile,
  createAutoBuildAdapter,
  createError,
  createStaticBuildAdapter,
  vitebookPlugin as default,
  endslash,
  ensureDir,
  ensureFile,
  escapeHTML,
  esmRequire,
  execRouteMatch,
  fixRegexInPath,
  formatDateString,
  getRouteMatchingPathname,
  hash,
  inBrowser,
  installURLPattern,
  isArray,
  isBoolean,
  isCommonJsFile,
  isFunction,
  isLinkExternal,
  isLinkHttp,
  isLinkMailto,
  isLinkTel,
  isLoadedMarkdownPage,
  isLoadedPage,
  isNull,
  isNumber,
  isObject,
  isRoutePathDynamic,
  isString,
  isSubpath,
  isTypeScriptFile,
  isUndefined,
  isWindows,
  logger,
  lowercaseFirstLetter,
  matchRoute,
  matchRouteInfo,
  mkdirp,
  ms,
  noendslash,
  noop,
  normalizePath,
  normalizeURL,
  noslash,
  parseDataAssetURL,
  prettyJsonStr,
  readRecentlyChangedFile,
  renderMarkdocToHTML,
  resolveAppConfig,
  resolveDataAssetID,
  resolveRelativePath,
  resolveRouteFromFilePath,
  resolveRoutePathFromUrl,
  resolveStaticRouteFromFilePath,
  rimraf,
  safeNotEqual,
  slash,
  slashedSplit,
  slashes,
  sortOrderedPageFiles,
  splitRoutePath,
  stripImportQuotesFromJson,
  stripImportQuotesRE,
  stripPageOrderFromPath,
  stripRouteInfoFromFilePath,
  stripRouteMetaFromFilePath,
  toPascalCase,
  trimExt,
  unescapeHTML,
  uppercaseFirstLetter,
  virtualAliases,
  virtualModuleId,
  virtualModuleRequestPath,
  vitebookPlugin as vitebook
};
